/**
 * @modification $Date: 2004/01/26 18:17:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafCheckBoxUI.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.plaf.basic.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import java.io.Serializable;

public class SlafCheckBoxUI
       extends BasicCheckBoxUI
{
  private static SlafCheckBoxUI checkboxUI=new SlafCheckBoxUI();

  public static ComponentUI createUI(JComponent b)
  {
    return checkboxUI;
  }
 
  public Icon createIcon()
  {
    return SlafManager.getIcon("CheckBox.icon");
  }

  public void installUI(JComponent c)
  {
    super.installUI(c);
    SlafUtils.setOpaque(c,false);
    SlafUtils.setRollover((AbstractButton)c);
    
    ((AbstractButton)c).setFocusPainted
	(SlafManager.getBoolean("CheckBox.focusPainted",true));
  }

  protected void paintBackground(Graphics g, AbstractButton b)
  {
    Rectangle r=SlafUtils.getFullRect  (b);
    int       t=SlafUtils.getStatus    (b);
    Color     c=SlafUtils.getBackground(b);
    SlafUtils.paintBackground(g,r,b,c,t);
  }
  
  public void update(Graphics g, JComponent c)
  {
    paintBackground(g,(AbstractButton)c);
    paint(g, c);
    SlafUtils.paintOver(g,(AbstractButton)c);
  }

  /**
   * paint the check box
   */
  public synchronized void paint(Graphics g, JComponent c)
  {
    AbstractButton b = (AbstractButton) c;
    // model=b.getModel();
	
    Font f;
    FontMetrics fm;
    Rectangle viewRect, iconRect, textRect;

    // int w=c.getWidth();
    // int h=c.getHeight();

    f=c.getFont();
    g.setFont(f);
    fm=g.getFontMetrics();

    viewRect=SlafUtils.getViewRect(b);
    iconRect=new Rectangle();
    textRect=new Rectangle();

    String text=SwingUtilities.layoutCompoundLabel
      (b,fm, b.getText(), getIcon(b),
       b.getVerticalAlignment(), b.getHorizontalAlignment(),
       b.getVerticalTextPosition(), b.getHorizontalTextPosition(),
       viewRect, iconRect, textRect, getDefaultTextIconGap(b));
	
    if(icon!=null)
      paintIcon(g,b,iconRect);

    if(text!=null)
      paintText(g,b,textRect,text);

    if(SlafUtils.hasFocus(b)&&b.isFocusPainted())
      paintFocus(g,b,viewRect,textRect,iconRect);
  }

  protected void paintFocus
    (Graphics g, AbstractButton b,
     Rectangle viewRect, Rectangle textRect, Rectangle iconRect)
  {
    SlafUtils.paintFocus(g,b,viewRect,textRect,iconRect);

    /*
    Rectangle r=SlafUtils.getViewRect(b);
    g.setColor(SlafManager.getColor("CheckBox.focus"));
    g.drawLine(r.x, r.y+r.height-1, r.x+r.width-1, r.y+r.height-1);
    */
  }

  protected Icon getIcon(AbstractButton b)
  {
    ButtonModel    m=b.getModel();
    Icon           a=null;

    if(!m.isEnabled())
      a=b.getDisabledIcon();
    else
    if(m.isPressed()&&m.isArmed())
    {
      a=b.getPressedIcon();
      if(a==null)
	a=b.getSelectedIcon();
    }
    else
    if(m.isSelected())
      a=b.getSelectedIcon();
    else
    if(b.isRolloverEnabled()&&m.isRollover())
      a=b.getRolloverIcon();
	      
    if(a==null)
      a=b.getIcon();

    if(a==null)
      a=icon;

    return a;
  }

  protected void paintIcon
    (Graphics g, AbstractButton b, Rectangle iconRect)
  {
    getIcon(b).paintIcon(b,g,iconRect.x,iconRect.y);
  }

  protected void paintText
    (Graphics g, AbstractButton b, Rectangle textRect, String text)
  {
    SlafUtils.drawString(g,text,
			 textRect.x,
			 textRect.y,"CheckBox",b);
    /*
    // AbstractButton b =(AbstractButton) c;			     
    ButtonModel    m =b.getModel();
    FontMetrics    fm=g.getFontMetrics();

    if(m.isEnabled())
    {
      Color fg=SlafUtils.getAdaptedForeground(b);
      g.setColor(fg);
      BasicGraphicsUtils.drawString(g,text, m.getMnemonic(),
				    textRect.x,
				    textRect.y+fm.getAscent());
    }
    else
    {
      Color df=SlafManager.getColor("CheckBox.disabledForeground");
      Color ds=SlafManager.getColor("CheckBox.disabledShadow");

      if(df!=null)
      {
	if(ds!=null)
	{
	  g.setColor(ds);
	  BasicGraphicsUtils.drawString(g,text,m.getMnemonic(),
					textRect.x+1,
					textRect.y+fm.getAscent()+1);
	}
	g.setColor(df);
	BasicGraphicsUtils.drawString(g,text,m.getMnemonic(),
				      textRect.x,
				      textRect.y+fm.getAscent());
      }
      else
      {
	df=SlafUtils.getAdaptedBackground(b);
	g.setColor(SlafUtils.brighten(df));
	BasicGraphicsUtils.drawString(g,text,m.getMnemonic(),
				      textRect.x+1,
				      textRect.y+fm.getAscent()+1);
	if(ds==null) ds=SlafUtils.darken(df);
	g.setColor(ds);
	BasicGraphicsUtils.drawString(g,text,m.getMnemonic(),
				      textRect.x,
				      textRect.y+fm.getAscent());
      }
    }
    */
  }
}

