/**
 * @modification $Date: 2004/01/26 18:17:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafComboBoxButton.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.plaf.basic.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.border.*;
import java.io.Serializable;

public class SlafComboBoxButton extends JButton
{
  protected JComboBox comboBox;
  protected JList listBox;
  protected CellRendererPane rendererPane;
  protected Icon comboIcon;
  protected boolean iconOnly = false;
  
  public final JComboBox getComboBox() { return comboBox;}
  public final void setComboBox( JComboBox cb ) { comboBox = cb;}

  public final Icon getComboIcon() { return comboIcon;}
  public final void setComboIcon( Icon i ) { comboIcon = i;}

  public final boolean isIconOnly() { return iconOnly;}
  public final void setIconOnly( boolean isIconOnly ) { iconOnly = isIconOnly;}

  SlafComboBoxButton()
  {
    super("");
    setOpaque(false);
    setRolloverEnabled(false);

    DefaultButtonModel model=new DefaultButtonModel()
    {
      public void setArmed(boolean armed)
	{ super.setArmed(isPressed() ? true : armed); }
    };
    setModel(model);
  }

  public SlafComboBoxButton(JComboBox cb, Icon i, 
			    CellRendererPane pane, JList list)
  {
    this();
    comboBox = cb;
    comboIcon = i;
    rendererPane = pane;
    listBox = list;
    
    setEnabled(comboBox.isEnabled());
    addFocusListener(new FocusGetter());

    try { setDefaultCapable(false); }
    catch(Throwable th) { }
  }

  public SlafComboBoxButton(JComboBox cb, Icon i, boolean onlyIcon,
			    CellRendererPane pane, JList list)
  {
    this(cb,i,pane,list);
    iconOnly=onlyIcon;
  }
    
  /*
  public boolean isFocusTraversable()
  {
    return !comboBox.isEditable();
  }
  */

  public void setBorder(Border b)
  {
    //super.setBorder(SlafManager.getBorder("ComboBox.buttonBorder"));

    b=SlafManager.getBorder
	("ComboBoxButton.border",
	 SlafManager.getBorder("ComboBox.buttonBorder"));

    if(SlafManager.getBorder("CombBoxButton.normalBorder")!=null)
	b=new SlafCompoundBorder
	    (new SlafExtraBorder("ComboBoxButton"),b,true);
    else
    if(SlafManager.getBorder("ComboBox.normalBorder")!=null)
	b=new SlafCompoundBorder
	    (new SlafExtraBorder("ComboBox"),b,true);

    if((b!=null)&&!(b instanceof BorderUIResource))
      b=new BorderUIResource(b);

    super.setBorder(b);
    setFocusPainted(false);
  }

  public void setOpaque(boolean b)
  {
    super.setOpaque(SlafManager.getBoolean
		    ("ComboBoxButton.opaque",
		     SlafManager.getBoolean
		     ("ComboBox.opaque",
		      SlafManager.getBoolean
		      ("Button.opaque",true))));//false)));
    if(!SlafManager.getBoolean("ComboBoxButton.texturePainted",true))
      putClientProperty("SLAF_NO_TEXTURE",Boolean.TRUE);
  }

  public void setRolloverEnabled(boolean b)
    { super.setRolloverEnabled
	(SlafManager.getBoolean("ComboBox.rollover",true)); }

  public Dimension getPreferredSize()
  {
    Dimension r=super.getPreferredSize();
    if(comboIcon!=null)
    {
      Insets insets=getInsets();
      int w=comboIcon.getIconWidth ()+(comboBox.isEditable() ? 2 : 6);
      int h=comboIcon.getIconHeight();
      r.width =Math.max(r.width ,w+insets.left+insets.right);
      r.height=Math.max(r.height,h+insets.top+insets.bottom);
    }
    return r;
  }

  public void paintComponent(Graphics g)
  {
    super.paintComponent(g);

    Insets insets = getInsets();

    int width = getWidth() - (insets.left + insets.right);
    int height = getHeight() - (insets.top + insets.bottom);

    int left = insets.left;
    int top = insets.top;
    int right = left + (width - 1);
    int bottom = top + (height - 1);
    
    int iconWidth = 0;
    int iconLeft = right;

    // Paint the icon
    if(comboIcon!=null)
    {
      iconWidth = comboIcon.getIconWidth();
      int iconHeight = comboIcon.getIconHeight();
      int iconTop = 0;
      
      if(iconOnly)
      {
	iconLeft = (left + (width - 1)) - iconWidth;
	iconTop = (top + ((bottom - top) / 2)) - (iconHeight / 2);
	// iconLeft = (getWidth() / 2) - (iconWidth / 2);
	// iconTop = (getHeight() / 2) - (iconHeight / 2);
      }
      else
      {
	iconLeft = (left + (width - 1)) - iconWidth;
	iconTop = (top + ((bottom - top) / 2)) - (iconHeight / 2);
	
	int xbar=insets.right+2;

	if(  comboBox.isEnabled()&&!comboBox.isEditable()
	   &&SlafManager.getBoolean("ComboBox.bar",false))
	{
	  g.setColor(getBackground());
	  //right=9
	  g.draw3DRect(iconLeft-xbar,top-2,1,height+3,
		       getModel().isPressed());
	}

	iconWidth+=xbar;
	//iconWidth+=9;
      }

      comboIcon.paintIcon( this, g, iconLeft, iconTop );
    }
	    
    int cWidth = width - (insets.right + iconWidth);

    // Paint the focus
    //if(hasFocus())
    //{
    //g.setColor( SlafLookAndFeel.getFocusColor() );
    //g.drawLine(left,height+3,left+cWidth-1,height+3);
    //}

    // Let the renderer paint
    if(!iconOnly&&(comboBox!=null))
      {
	ListCellRenderer renderer=comboBox.getRenderer();
	Component c;
	boolean renderPressed = getModel().isPressed();
	c=renderer.getListCellRendererComponent(listBox,
						comboBox.getSelectedItem(),
						-1,
						false, // renderPressed,
						false);
	c.setFont(rendererPane.getFont());
	c.setEnabled(comboBox.isEnabled());

	//if(model.isArmed()&&model.isPressed())
	//{
	//if(isOpaque())
	//c.setBackground(SlafManager.getColor("Button.pressed"));
	//}
	//else

	c.setBackground(comboBox.getBackground());
	c.setForeground(comboBox.getForeground());

	// NEW @GDX
	JComponent jc=(JComponent)c;
	boolean opaque=jc.isOpaque();
	jc.setOpaque(false);
	rendererPane.paintComponent( g, c, this, left, top, cWidth, height );
	jc.setOpaque(opaque);
      }
  }

  protected class FocusGetter
      extends FocusAdapter implements Serializable
  {
      public void focusGained(FocusEvent e)
      { 
	//if(comboBox.isEditable()) transferFocus();
      }
            
      public void focusLost(FocusEvent e)
      {
	if(!comboBox.isEditable()) comboBox.hidePopup();
      }
  }
}

