/**
 * @modification $Date: 2004/05/11 19:08:41 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafDiscreteFilter.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.image.*;

public class SlafDiscreteFilter extends RGBImageFilter
{
  private int   n_;
  private int[] r_,g_,b_,c_;

  public SlafDiscreteFilter()
  {
    n_=0;
    r_=new int[128];
    g_=new int[128];
    b_=new int[128];
    c_=new int[128];
  }

  public void addColor(int rgb)
  {
    r_[n_]=(rgb & 0x00ff0000) >>16;
    g_[n_]=(rgb & 0x0000ff00) >> 8;
    b_[n_]=(rgb & 0x000000ff)     ;
    c_[n_]=(rgb & 0x00ffffff)     ;
    n_++;
  }

  private int findClosest(int _r,int _g,int _b)
  {
    int dmin=Integer.MAX_VALUE;
    int imin=0;

    for(int i=0;i<n_;i++)
    {
      int d=
        (_r-r_[i])*(_r-r_[i])+
        (_g-g_[i])*(_g-g_[i])+
        (_b-b_[i])*(_b-b_[i]);
      if(d<dmin)
      {
        dmin=d;
        imin=i;
      }
    }

    return imin;    
  }
  
  public int filterRGB(int x, int y, int rgb)
  {
    int r=(rgb & 0x00ff0000) >>16;
    int g=(rgb & 0x0000ff00) >> 8;
    int b=(rgb & 0x000000ff)     ;
    int i=findClosest(r,g,b);

    /*
    System.err.println(i+": "+Integer.toHexString(rgb & 0x00ffffff)+
                       "-->"+Integer.toHexString(c_[i]));
    System.err.println(r+","+g+","+b);
    */
    return (rgb & 0xff000000) | c_[i];
  }
}
