/**
 * @modification $Date: 2004/01/26 18:17:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafImageBorder.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;

public class SlafImageBorder extends AbstractBorder
{
  protected int[] margin;
  protected int[] thickness;
  protected Image image;

  public SlafImageBorder(Insets _t, Image _img)
  {
    this(0,0,0,0,_t.top,_t.left,_t.bottom,_t.right,_img);
  }

  public SlafImageBorder(int _m1, int _m2,int _m3,int _m4,
			int _t1, int _t2,int _t3,int _t4,
			Image _img)
  {
    margin   =new int[] { _m1,_m2,_m3,_m4 };
    thickness=new int[] { _t1,_t2,_t3,_t4 };
    image=_img;
  }

  public void paintBorder(Component c, Graphics g,
			  int x, int y, int width, int height)
  {
    if(image==null)
    {
      SlafUtils.error("SlafBorder: image==null");
      return;
    }

    int top   =thickness[0];
    int left  =thickness[1];
    int bottom=thickness[2];
    int right =thickness[3];

    Image im=image;

    Color bg=c.getBackground();
    if((bg!=null)
       &&!(bg instanceof ColorUIResource)
       &&(c instanceof JComponent))
    {
      String ui=((JComponent)c).getUIClassID();
      if(ui.endsWith("UI")) ui=ui.substring(0,ui.length()-2);
      if(SlafManager.getInteger(ui+".textureType",-1)==-1)
	im=SlafUtils.mixBackground((JComponent)c,im,bg,true,255);
    }

    SlafUtils.paintImage(im,c,g,
			 new Rectangle(x,y,width,height),
			 new Insets(top,left,bottom,right),
			 false);
  }

  public Insets getBorderInsets(Component _c)
  {
    return getBorderInsets(_c,new Insets(0,0,0,0));
  }

  public Insets getBorderInsets(Component _c, Insets _insets)
  {
    _insets.top   =margin[0]+thickness[0];
    _insets.left  =margin[1]+thickness[1];
    _insets.bottom=margin[2]+thickness[2];
    _insets.right =margin[3]+thickness[3];

    return _insets;
  }

  public boolean isBorderOpaque() { return false; }
}
