/**
 * @modification $Date: 2004/06/11 19:23:24 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafLookAndFeel.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import javax.swing.border.*;
import javax.swing.text.*;

public class SlafLookAndFeel
       extends BasicLookAndFeel
{
  public static SlafDefaultTheme currentTheme;

  public SlafLookAndFeel()
  {
    this(null);
  }

  public SlafLookAndFeel(String _theme)
  {
    if((_theme==null)&&(currentTheme==null))
    {
      try { _theme=System.getProperty("slaf.theme"); }
      catch(SecurityException ex) { }
    }
    if(_theme!=null) setCurrentTheme(_theme);
  }

  public String getName()
  {
    return "Slaf";
  }

  public String getID()
  {
    return "Slaf";
  }

  public String getDescription()
  {
    return "The Simple Look and Feel";
  }
    
  public boolean isNativeLookAndFeel()
  {
    return false;
  }

  public boolean isSupportedLookAndFeel()
  {
    return true;
  }
  
  public boolean getSupportsWindowDecorations()
  {
    return true;
  }

  protected void initClassDefaults(UIDefaults table)
  {
    //SlafUtils.trace("ClassDefaults");
    super.initClassDefaults(table);

    String basicPackageName = "javax.swing.plaf.basic.";
    String metalPackageName = "javax.swing.plaf.metal.";
    String slafPackageName  = "com.memoire.slaf.";

    String[] uis=
    {
                   "ButtonUI", slafPackageName + "SlafButtonUI",
                 "CheckBoxUI", slafPackageName + "SlafCheckBoxUI",
             "ColorChooserUI", slafPackageName + "SlafColorChooserUI",
                  "MenuBarUI", slafPackageName + "SlafMenuBarUI",
                     "MenuUI", slafPackageName + "SlafMenuUI",
                 "MenuItemUI", slafPackageName + "SlafMenuItemUI",
         "CheckBoxMenuItemUI", slafPackageName + "SlafCheckBoxMenuItemUI",
      "RadioButtonMenuItemUI", slafPackageName + "SlafRadioButtonMenuItemUI",
              "RadioButtonUI", slafPackageName + "SlafRadioButtonUI",
             "ToggleButtonUI", slafPackageName + "SlafToggleButtonUI",
                "PopupMenuUI", slafPackageName + "SlafPopupMenuUI",
              "ProgressBarUI", slafPackageName + "SlafProgressBarUI",
                "ScrollBarUI", slafPackageName + "SlafScrollBarUI",
               "ScrollPaneUI", slafPackageName + "SlafScrollPaneUI",
                "SplitPaneUI", slafPackageName + "SlafSplitPaneUI",
                   "SliderUI", slafPackageName + "SlafSliderUI",
                "SeparatorUI", slafPackageName + "SlafSeparatorUI",
         "ToolBarSeparatorUI", slafPackageName + "SlafToolBarSeparatorUI",
       "PopupMenuSeparatorUI", slafPackageName + "SlafPopupMenuSeparatorUI",
	       "TabbedPaneUI", slafPackageName + "SlafTabbedPaneUI",
                 "TextAreaUI", slafPackageName + "SlafTextAreaUI",
                "TextFieldUI", slafPackageName + "SlafTextFieldUI",
            "PasswordFieldUI", slafPackageName + "SlafPasswordFieldUI",
                 "TextPaneUI", slafPackageName + "SlafTextPaneUI",
               "EditorPaneUI", slafPackageName + "SlafEditorPaneUI",
                     "TreeUI", slafPackageName + "SlafTreeUI",
                    "LabelUI", slafPackageName + "SlafLabelUI",
                     "ListUI", slafPackageName + "SlafListUI",
                  "ToolBarUI", slafPackageName + "SlafToolBarUI",
                  "ToolTipUI", slafPackageName + "SlafToolTipUI",
		 "ComboBoxUI", slafPackageName + "SlafComboBoxUI",
                    "TableUI", slafPackageName + "SlafTableUI",
              "TableHeaderUI", slafPackageName + "SlafTableHeaderUI",
            "InternalFrameUI", slafPackageName + "SlafInternalFrameUI",
              "DesktopPaneUI", slafPackageName + "SlafDesktopPaneUI",
              "DesktopIconUI", slafPackageName + "SlafDesktopIconUI",
	       "OptionPaneUI", slafPackageName + "SlafOptionPaneUI",
		    "PanelUI", slafPackageName + "SlafPanelUI",
		 "ViewportUI", slafPackageName + "SlafViewportUI",
		 "RootPaneUI", slafPackageName + "SlafRootPaneUI",

		   "FileChooserUI", slafPackageName + "SlafFileChooserUI",
		   //"FileChooserUI", metalPackageName+ "MetalFileChooserUI",
		   //"FileChooserUI", basicPackageName+ "BasicFileChooserUI",

	 //"StandardDialogUI", basicPackageName + "BasicStandardDialogUI",

		   // SLAF only
             "ScrollButtonUI", slafPackageName + "SlafScrollButtonUI",
 "InternalFrameTitleButtonUI", slafPackageName + "SlafInternalFrameTitleButtonUI",

		   // JDK 1.4
		  "SpinnerUI", slafPackageName + "SlafSpinnerUI",
       "FormattedTextFieldUI", slafPackageName + "SlafFormattedTextFieldUI",
    };
    
    table.putDefaults(uis);

    /*
    ClassLoader cl=getClass().getClassLoader();
    table.put("ClassLoader",cl);
    for(int i=0;i<uis.length-1;i+=2)
    {
      String cn=uis[i+1];
      Class  cc=null;
      try
      {
	if(cl==null) cc=Class.forName(cn);
	else         cc=cl.loadClass(cn);
	if(cc!=null) table.put(cn,cc);
      }
      catch(Throwable ex) // ClassNotFoundException ex
	{ SlafUtils.trace("Class "+cn+" not found"); }
    }
    */

    //SlafUtils.trace("TABLE="+table.size());
  }

  /**
   * Load the SystemColors into the defaults table.  The keys
   * for SystemColor defaults are the same as the names of
   * the public fields in SystemColor.
   */
  protected void initSystemColorDefaults(UIDefaults table)
  {
    //SlafUtils.trace("SystemColorDefaults");
    super.initSystemColorDefaults(table);

    ColorUIResource black=new ColorUIResource(Color.blue);//255,255,0);

    Object[] colors=
    {
                "desktop", black,
          "activeCaption", black,
      "activeCaptionText", black,
    "activeCaptionBorder", black,
        "inactiveCaption", black,
    "inactiveCaptionText", black,
  "inactiveCaptionBorder", black,
                 "window", black,
           "windowBorder", black,
             "windowText", black,
                   "menu", black,
               "menuText", black,
                   "text", black,
               "textText", black,
          "textHighlight", black,
      "textHighlightText", black,
       "textInactiveText", black,
                "control", black,
            "controlText", black,
       "controlHighlight", black,
     "controlLtHighlight", black,
          "controlShadow", black,
        "controlDkShadow", black,
              "scrollbar", black,
		   "info", black,
               "infoText", black,
    };
    
    table.putDefaults(colors);

    //SlafUtils.trace("TABLE="+table.size());
  }

  protected void initComponentDefaults(UIDefaults table)
  {
    //SlafUtils.trace("ComponentDefaults");
    super.initComponentDefaults(table);

    if(SlafUtils.jdk()>=1.3)
      SlafFeelNew.install(table);
    else
      SlafFeelOld.install(table);

	// Editors

        Object comboBoxEditorActiveValue=new UIDefaults.ActiveValue()
	{
	  public Object createValue(UIDefaults _table)
	  {
	    return new SlafComboBoxEditor();
	  }
	};

	// Renderers

        Object comboBoxCellRendererActiveValue=new UIDefaults.ActiveValue()
	{
	  public Object createValue(UIDefaults _table)
	  {
	    return new SlafComboBoxCellRenderer();
	  }
	};

        Object listCellRendererActiveValue=new UIDefaults.ActiveValue()
	{
	  public Object createValue(UIDefaults _table)
	  {
	    return new SlafListCellRenderer();
	  }
	};

        Object tableCellRendererActiveValue=new UIDefaults.ActiveValue()
	{
	  public Object createValue(UIDefaults _table)
	  {
	    return new SlafTableCellRenderer();
	  }
	};

        Object treeCellRendererActiveValue=new UIDefaults.ActiveValue()
	{
	  public Object createValue(UIDefaults _table)
	  {
	    return new SlafTreeCellRenderer();
	  }
	};

        //
        // DEFAULTS TABLE
        //

	//final int internalFrameIconSize = 12;
	//final int scrollBarSize         = 12;

        Object[] defaults = {

            // CheckBox
            "CheckBox.icon",            SlafIconFactory.getCheckBoxIcon(),
            "CheckBox.selectedIcon",    SlafIconFactory.getCheckBoxSelectedIcon(),
            "CheckBox.unselectedIcon",  SlafIconFactory.getCheckBoxUnselectedIcon(),

            // File View 
            "FileView.directoryIcon",   SlafIconFactory.getTreeFolderOpenIcon(),
            "FileView.fileIcon",        SlafIconFactory.getTreeLeafIcon(),
            "FileView.computerIcon",    SlafIconFactory.getTreeComputerIcon(),
            "FileView.hardDriveIcon",   SlafIconFactory.getTreeHardDriveIcon(),
            "FileView.floppyDriveIcon", SlafIconFactory.getTreeFloppyDriveIcon(),

            // File Chooser
            "FileChooser.detailsViewIcon", SlafIconFactory.getFileChooserDetailViewIcon(),
            "FileChooser.homeFolderIcon",  SlafIconFactory.getFileChooserHomeFolderIcon(),
            "FileChooser.listViewIcon",    SlafIconFactory.getFileChooserListViewIcon(),
            "FileChooser.newFolderIcon",   SlafIconFactory.getFileChooserNewFolderIcon(),
            "FileChooser.upFolderIcon",    SlafIconFactory.getFileChooserUpFolderIcon(),

	    // OptionPane
	    "OptionPane.errorIcon",        SlafIconFactory.getOptionPaneErrorIcon(),
	    "OptionPane.informationIcon",  SlafIconFactory.getOptionPaneInformationIcon(),
	    "OptionPane.warningIcon",      SlafIconFactory.getOptionPaneWarningIcon(),
	    "OptionPane.questionIcon",     SlafIconFactory.getOptionPaneQuestionIcon(),

            // RadioButton
            "RadioButton.icon",            SlafIconFactory.getRadioButtonIcon(),
            "RadioButton.selectedIcon",    SlafIconFactory.getRadioButtonSelectedIcon(),
            "RadioButton.unselectedIcon",  SlafIconFactory.getRadioButtonUnselectedIcon(),

            // "Slider.horizontalThumbIcon", SlafIconFactory.getHorizontalSliderThumbIcon(),
            // "Slider.verticalThumbIcon",   SlafIconFactory.getVerticalSliderThumbIcon(),

            "ComboBox.editor",                 comboBoxEditorActiveValue,
            "ComboBox.icon",                   SlafIconFactory.getComboBoxIcon(),
            "ComboBox.renderer",               comboBoxCellRendererActiveValue,

            "InternalFrame.icon",             SlafIconFactory.getInternalFrameDefaultMenuIcon(),

            // InternalFrame TitlePane
            "InternalFrameTitlePane.linifyIcon",      SlafIconFactory.getInternalFrameLinifyIcon(),//internalFrameIconSize),
            "InternalFrameTitlePane.closeIcon",       SlafIconFactory.getInternalFrameCloseIcon(),//internalFrameIconSize),
            "InternalFrameTitlePane.maximizeIcon",    SlafIconFactory.getInternalFrameMaximizeIcon(),//internalFrameIconSize),
            "InternalFrameTitlePane.iconizeIcon",     SlafIconFactory.getInternalFrameMinimizeIcon(),//internalFrameIconSize),
            "InternalFrameTitlePane.altMaximizeIcon", SlafIconFactory.getInternalFrameAltMaximizeIcon(),//internalFrameIconSize),

            "List.cellRenderer", listCellRendererActiveValue,

	    // ScrollButton
            "ScrollButton.northIcon",          SlafIconFactory.getNorthIcon(),//scrollBarSize),
            "ScrollButton.southIcon",          SlafIconFactory.getSouthIcon(),//scrollBarSize),
            "ScrollButton.eastIcon",           SlafIconFactory.getEastIcon (),//scrollBarSize),
            "ScrollButton.westIcon",           SlafIconFactory.getWestIcon (),//scrollBarSize),

            "Menu.arrowIcon",                      SlafIconFactory.getMenuArrowIcon(),
            "Menu.checkIcon",                      SlafIconFactory.getMenuCheckIcon(),
            "MenuItem.arrowIcon",                  SlafIconFactory.getMenuItemArrowIcon(),
            "MenuItem.checkIcon",                  SlafIconFactory.getMenuItemCheckIcon(),

            // CB & RB Menu Item
            "CheckBoxMenuItem.arrowIcon",          SlafIconFactory.getCheckBoxMenuItemArrowIcon(),
            "CheckBoxMenuItem.checkIcon",          SlafIconFactory.getCheckBoxMenuItemCheckIcon(),
            "CheckBoxMenuItem.selectedIcon",       SlafIconFactory.getCheckBoxSelectedIcon(),
            "CheckBoxMenuItem.unselectedIcon",     SlafIconFactory.getCheckBoxUnselectedIcon(),
            "RadioButtonMenuItem.arrowIcon",       SlafIconFactory.getRadioButtonMenuItemArrowIcon(),
            "RadioButtonMenuItem.checkIcon",       SlafIconFactory.getRadioButtonMenuItemCheckIcon(),
            "RadioButtonMenuItem.selectedIcon",    SlafIconFactory.getRadioButtonSelectedIcon(),
            "RadioButtonMenuItem.unselectedIcon",  SlafIconFactory.getRadioButtonUnselectedIcon(),

            "Table.cellRenderer",               tableCellRendererActiveValue,

            "Tree.openIcon",                    SlafIconFactory.getTreeFolderOpenIcon(),
            "Tree.closedIcon",                  SlafIconFactory.getTreeFolderClosedIcon(),
            "Tree.leafIcon",                    SlafIconFactory.getTreeLeafIcon(),
            "Tree.expandedIcon",                SlafIconFactory.getTreeControlIcon(SlafIconFactory.DARK),
            "Tree.collapsedIcon",               SlafIconFactory.getTreeControlIcon(SlafIconFactory.LIGHT),
            // "Tree.line",                        getPrimaryControl(), // horiz lines
            // "Tree.hash",                        getPrimaryControl(), // legs
            "Tree.cellRenderer",                treeCellRendererActiveValue,
        };

      table.putDefaults(defaults);
      //SlafUtils.trace("TABLE="+table.size());
    }

    public UIDefaults getDefaults()
    {
      if(currentTheme==null)
      {
	SlafUtils.trace("create default theme");
	currentTheme=new SlafDefaultTheme();
      }

      UIDefaults table=super.getDefaults();

      if(table.get("ClassLoader")==null)
        table.put("ClassLoader",getClass().getClassLoader());

      //SlafUtils.trace("add custom entries");
      currentTheme.addCustomEntriesToTable(table);

      //SlafUtils.trace("TABLE="+table.size());
      return table;
    }

    public void setCurrentTheme(SlafDefaultTheme theme)
    {
      if(theme==null)
	throw new NullPointerException("Can't have null theme");

      SlafUtils.clearCache();
      currentTheme=theme;
      //currentTheme.addCustomEntriesToTable(getDefaults());
    }

    public static String getCurrentTheme()
    {
      return currentTheme==null ? "default" : currentTheme.getName();
    }

    public static void setCurrentTheme(String key)
    {
      if(key!=null) key=key.toLowerCase();

      SlafDefaultTheme theme;
      /*
      if((key!=null)&&key.startsWith("gtk:"))
	theme=new SlafGtkTheme(key.substring(4));
      else
      */
      theme=new SlafCustomTheme(key);
      currentTheme=theme;
      //currentTheme.addCustomEntriesToTable
      //(UIManager.getLookAndFeelDefaults());
    }

  public static ColorUIResource  getFocusColor()
    { return currentTheme.getColorResource("Focus.foreground"); }

  public static ColorUIResource  getCaretColor()
    { return currentTheme.getColorResource("Caret.foreground"); }

  public static ColorUIResource  getUserTextColor()
    { return currentTheme.getColorResource("User.foreground"); }

  public static UIResource       getResource      (String _key) { return currentTheme.getResource(_key); }
  public static BorderUIResource getBorderResource(String _key) { return currentTheme.getBorderResource(_key); }
  public static ColorUIResource  getColorResource (String _key) { return currentTheme.getColorResource(_key); }
  public static FontUIResource   getFontResource  (String _key) { return currentTheme.getFontResource(_key); }

  public static FontUIResource  getControlTextFont() { return currentTheme.getFontResource("Component.font"); }
  public static FontUIResource  getSystemTextFont()  { return currentTheme.getFontResource("System.font"); }
  public static FontUIResource  getUserTextFont()    { return currentTheme.getFontResource("User.font"); }
  public static FontUIResource  getWindowTitleFont() { return currentTheme.getFontResource("Window.titleFont"); }
  public static FontUIResource  getSubTextFont()     { return currentTheme.getFontResource("Small.font"); }
  public static FontUIResource  getMenuTextFont()    { return currentTheme.getFontResource("Component.font"); }

}
