/**
 * @modification $Date: 2004/01/26 18:17:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafRadioButtonUI.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.plaf.basic.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import java.io.Serializable;

public class SlafRadioButtonUI
       extends BasicRadioButtonUI
{
  private static SlafRadioButtonUI radioButtonUI=new SlafRadioButtonUI();

  public static ComponentUI createUI(JComponent b)
  {
    return radioButtonUI;
  }

  public Icon createIcon()
  {
    return SlafManager.getIcon("RadioButton.icon");
  }

  public void installUI(JComponent c)
  {
    super.installUI(c);
    SlafUtils.setOpaque(c,false);
    SlafUtils.setRollover((AbstractButton)c);

    ((AbstractButton)c).setFocusPainted
	(SlafManager.getBoolean("RadioButton.focusPainted",true));
  }

  protected void paintBackground(Graphics g, AbstractButton b)
  {
    Rectangle r=SlafUtils.getFullRect  (b);
    int       t=SlafUtils.getStatus    (b);
    Color     c=SlafUtils.getBackground(b);
    SlafUtils.paintBackground(g,r,b,c,t);
  }
  
  public void update(Graphics g, JComponent c)
  {
    paintBackground(g,(AbstractButton)c);
    paint(g, c);
    SlafUtils.paintOver(g,(AbstractButton)c);
  }

  /**
   * paint the radio button
   */
  public synchronized void paint(Graphics g, JComponent c)
  {
    AbstractButton b = (AbstractButton) c;
    // model=b.getModel();
	
    Font f;
    FontMetrics fm;
    Rectangle viewRect, iconRect, textRect;

    // int w=c.getWidth();
    // int h=c.getHeight();

    f=c.getFont();
    g.setFont(f);
    fm=g.getFontMetrics();

    viewRect=SlafUtils.getViewRect(b);
    iconRect=new Rectangle();
    textRect=new Rectangle();

    String text=SwingUtilities.layoutCompoundLabel
      (b,fm, b.getText(), getIcon(b),
       b.getVerticalAlignment(), b.getHorizontalAlignment(),
       b.getVerticalTextPosition(), b.getHorizontalTextPosition(),
       viewRect, iconRect, textRect, getDefaultTextIconGap(b));
	
    if(icon!=null)
      paintIcon(g,b,iconRect);

    if(text!=null)
      paintText(g,b,textRect,text);

    if(SlafUtils.hasFocus(b)&&b.isFocusPainted())
      paintFocus(g,b,viewRect,textRect,iconRect);
  }

  protected void paintFocus
    (Graphics g, AbstractButton b,
     Rectangle viewRect, Rectangle textRect, Rectangle iconRect)
  {
    SlafUtils.paintFocus(g,b,viewRect,textRect,iconRect);

    /*
    Rectangle r=SlafUtils.getViewRect(b);
    g.setColor(SlafManager.getColor("RadioButton.focus"));
    g.drawLine(r.x, r.y+r.height-1, r.x+r.width-1, r.y+r.height-1);
    */
  }

  protected Icon getIcon(AbstractButton b)
  {
    ButtonModel    m=b.getModel();
    Icon           a=null;

    if(!m.isEnabled())
      a=b.getDisabledIcon();
    else
    if(m.isPressed()&&m.isArmed())
    {
      a=b.getPressedIcon();
      if(a==null)
	a=b.getSelectedIcon();
    }
    else
    if(m.isSelected())
      a=b.getSelectedIcon();
    else
    if(b.isRolloverEnabled()&&m.isRollover())
      a=b.getRolloverIcon();
	      
    if(a==null)
      a=b.getIcon();

    if(a==null)
      a=icon;

    return a;
  }

  protected void paintIcon
    (Graphics g, AbstractButton b, Rectangle iconRect)
  {
    getIcon(b).paintIcon(b,g,iconRect.x,iconRect.y);
  }

  protected void paintText
    (Graphics g, AbstractButton b, Rectangle textRect, String text)
  {
    SlafUtils.drawString(g,text,
			 textRect.x,
			 textRect.y,"RadioButton",b);
  }
}

