/**
 * @modification $Date: 2004/06/11 19:23:24 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafResource.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.image.*;
import java.net.*;
import javax.swing.*;

public class SlafResource
{
  private static final Frame        HELPER =new Frame();
  private static final MediaTracker TRACKER=new MediaTracker(HELPER);
  private static final Image        DEFAULT=createDefaultImage();

  private static int trackerID_=0;

  private static final Image createDefaultImage()
  {
    HELPER.addNotify();

    final byte[] DB=new byte[] {
    (byte)0x47,(byte)0x49,(byte)0x46,(byte)0x38,(byte)0x39,(byte)0x61,
    (byte)0x10,(byte)0x00,(byte)0x10,(byte)0x00,(byte)0x80,(byte)0x00,
    (byte)0x00,(byte)0x00,(byte)0x00,(byte)0xFF,(byte)0xFF,(byte)0xFF,
    (byte)0xFF,(byte)0x21,(byte)0xF9,(byte)0x04,(byte)0x01,(byte)0x0A,
    (byte)0x00,(byte)0x01,(byte)0x00,(byte)0x2C,(byte)0x00,(byte)0x00,
    (byte)0x00,(byte)0x00,(byte)0x10,(byte)0x00,(byte)0x10,(byte)0x00,
    (byte)0x00,(byte)0x02,(byte)0x1A,(byte)0x44,(byte)0x8E,(byte)0xA9,
    (byte)0x61,(byte)0xEB,(byte)0xD7,(byte)0xDE,(byte)0x8A,(byte)0x32,
    (byte)0xD1,(byte)0x0A,(byte)0xB1,(byte)0xBA,(byte)0x98,(byte)0x57,
    (byte)0x2F,(byte)0x81,(byte)0x8F,(byte)0xE8,(byte)0x90,(byte)0x93,
    (byte)0x66,(byte)0xA1,(byte)0x88,(byte)0xB9,(byte)0x15,(byte)0x00,
    (byte)0x3B
    };

    return HELPER.getToolkit().createImage(DB);
  }

  /**
   * Ensures that an image is loaded.
   * @param _image the image to load
   * @param _image an image to return if there is an error
   */
  public static final Image ensureImageIsLoaded(Image _image, Image _default)
  {
    if((_image==null)||(_image==_default)) return _default;
    
    int id;

    synchronized (TRACKER)
    {
      id=trackerID_;
      trackerID_++;
    }

    TRACKER.addImage(_image,id);
    TRACKER.checkID(id,true);

    //try { TRACKER.waitForID(id,15000l); }
    //catch(InterruptedException ex) { }

    long before,after,delay;
    before=System.currentTimeMillis();
    while((after=System.currentTimeMillis())-before<10000l)
    {
      try
      {
	delay=10000l-after+before;
	if(delay>50l) TRACKER.waitForID(id,delay);
	break;
      }
      catch(InterruptedException ex) { }
    }

    Image r;
    if(  TRACKER.isErrorID(id)
       ||((TRACKER.statusID(id,false)&TRACKER.LOADING)!=0))
      r=_default;
    else
      r=_image;

    if(r!=_image)
    {
      SlafUtils.trace("Failed to load image <"+id+">");
      //try { throw new RuntimeException("Slaf Media Tracker"); }
      //catch(RuntimeException ex) { ex.printStackTrace(); }
    }

    return r;
  }

  public static final SlafResource SLAF=new SlafResource();

  private int defaultSize_=16;
  public int  getDefaultSize()       { return defaultSize_; }
  public void setDefaultSize(int _v) { defaultSize_=_v; }

  private URL defaultBase_=null;
  public URL  getDefaultBase()       { return defaultBase_; }
  public void setDefaultBase(URL _v) { defaultBase_=_v; }

  // URL

  public URL getURL(String _path)
  {
    URL url=null;
    String p=_path;

    try
    {
      p=adjust(p);
      // SlafUtils.trace("P="+p);

      if(defaultBase_!=null)
      {
	try { url=new URL(p); }
	catch(MalformedURLException ex) { url=null; }
	// SlafUtils.trace("H="+url.getHost());
      }
      else
      {
	url=getClass().getResource(p);
	if(url==null)
	{
	  int i=p.lastIndexOf('.');
	  if(i>=0) p=p.substring(0,i)+"_"+getDefaultSize()+p.substring(i);
	  url=getClass().getResource(p);
	}
      }
    }
    catch(Exception ex) { url=null; }

    // SlafUtils.trace("U="+url);
    return url;
  }

  public String adjust(String _path)
  {
    String r=_path;

    // SlafUtils.trace("base:  "+defaultBase_);

    if(defaultBase_!=null)
    {
      // SlafUtils.trace("avant: "+_path);

      String c=getClass().getName();
      int i=c.lastIndexOf('.');
      if(i>=0) c=c.substring(0,i+1);
      c=c.replace('.','/');

      r=""+defaultBase_+c+_path;
      // SlafUtils.trace("apres: "+r);
    }

    return r;
  }

  // Icon

  public Icon getIcon(String _path, int _size)
  {
    String p=_path.toLowerCase();

    if(p.endsWith(".gif"))
      p=p.substring(0,p.length()-4)+"_"+_size+".gif";
    else
    if(p.endsWith(".jpg"))
      p=p.substring(0,p.length()-4)+"_"+_size+".jpg";
    else
    if(p.endsWith(".png"))
      p=p.substring(0,p.length()-4)+"_"+_size+".png";
    else
      p+="_"+_size+".gif";

    return getIcon(getURL(p));
  }

  /*
  public Icon getIcon(String _path)
  {
    String p=_path.toLowerCase();
    if(!p.endsWith(".gif")) p+=".gif";
    return getIcon(getURL(p));
  }
  */

  public ImageIcon getIcon(String _path)
  {
    Image r=getImage(_path);
    return (r==null) ? null : new ImageIcon(r);
  }

  public ImageIcon getIcon(URL _url)
  {
    Image r=getImage(_url);
    return (r==null) ? null : new ImageIcon(r);
  }

  public Image getImage(String _path)
  {
    if(_path==null) return null;
    return processImage(HELPER.getToolkit().getImage(_path));
  }
    
  public Image getImage(URL _url)
  {
    if(_url==null) return null;
    return processImage(HELPER.getToolkit().getImage(_url));
  }

  public static final Image processImage(Image r)
  {
    r=ensureImageIsLoaded(r,DEFAULT);

    if(SlafManager.getBoolean("Theme.applyIconFilter",true))
    {
      ImageFilter f=(ImageFilter)SlafManager.getObject("Theme.iconFilter");

      if(f!=null)
      {
        r=ensureImageIsLoaded
          (HELPER.createImage
           (new FilteredImageSource
            (r.getSource(),f)),r);
      }
    }

    return r;
  }
}
