/**
 * @modification $Date: 2004/01/26 18:17:33 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafFrameBorder.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.basic.*;
import javax.swing.plaf.*;

public class SlafRootPaneBorder
  extends AbstractBorder
{
  private String ui_;
  private Insets insets_;
  private Border enabled;
  private Border selected;
  private Border disabled;

  // public SlafRootPaneBorder()
  // { this(null); }

  public SlafRootPaneBorder(String _ui)
  {
    ui_=_ui;
  }

  private void setBorders(Component _c)
  {
    if(insets_!=null) return;

    final Container p=_c.getParent();
    if(ui_==null)
    {
           if(p instanceof JFrame ) ui_="Frame";
      else if(p instanceof JDialog) ui_="Dialog";
      else                          ui_="RootPane";
    }

    if(p instanceof Window)//!"RootPane".equals(ui_))
      ((Window)p).addWindowListener
	(new WindowAdapter()
	  {
	    public void windowActivated(WindowEvent _evt)
	    {
	      //System.err.println("ACTIVATED");
	      p.repaint();
	    }
	    public void windowDeactivated(WindowEvent _evt)
	    {
	      //System.err.println("DEACTIVATED");
	      p.repaint();
	    }
	  });
  
    /*
    if(ui_==null)
    {
      ui_=((JComponent)_c).getUIClassID();
      if(ui_.endsWith("UI")) ui_=ui_.substring(0,ui_.length()-2);
    }
    */

    // SlafUtils.trace("%%% "+ui_);
    if(enabled ==null) enabled =SlafManager.getBorder(ui_+".enabledBorder");
    if(selected==null) selected=SlafManager.getBorder(ui_+".selectedBorder");
    if(disabled==null) disabled=SlafManager.getBorder(ui_+".disabledBorder");

    if(enabled ==null) enabled =new EmptyBorder(0,0,0,0);
    //new LineBorder(Color.red,3);//yellow,2);
    if(selected==null) selected=enabled;
    if(disabled==null) disabled=enabled;

    Insets ebi=enabled .getBorderInsets(_c);
    Insets sbi=selected.getBorderInsets(_c);
    Insets dbi=disabled.getBorderInsets(_c);

    int ti=SlafUtils.max(ebi.top   ,sbi.top   ,dbi.top   );
    int bi=SlafUtils.max(ebi.bottom,sbi.bottom,dbi.bottom);
    int li=SlafUtils.max(ebi.left  ,sbi.left  ,dbi.left  );
    int ri=SlafUtils.max(ebi.right ,sbi.right ,dbi.right );

    insets_=new Insets(ti,li,bi,ri);
  }

  public void paintBorder(Component c, Graphics g, int x, int y, int w, int h)
  {
    setBorders(c);

    if(!(c instanceof JRootPane))
    {
      SlafUtils.error("c is not a RootPane [SlafRootPaneBorder.java]");
      SlafUtils.error("c is a "+c.getClass());
      return;
    }

    Border b=(c.isEnabled() ? enabled : disabled);

    Container p=c.getParent();
    if(p instanceof Window)
    {
      //if(((Window)p).isFocused()) // JDK1.4
      if(((Window)p).getFocusOwner()!=null)
	b=selected;
      else
	b=enabled;
    }

    b.paintBorder(c,g,x,y,w,h);
  }

  public Insets getBorderInsets(Component _c)
  {
    return getBorderInsets(_c,new Insets(0,0,0,0));
  }

  public Insets getBorderInsets(Component _c, Insets _insets)
  {
    setBorders(_c);

    _insets.top   =insets_.top;
    _insets.left  =insets_.left;
    _insets.bottom=insets_.bottom;
    _insets.right =insets_.right;

    return _insets;
  }
}
