/**
 * @modification $Date: 2004/06/14 19:33:10 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafLookAndFeel.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import javax.swing.border.*;
import javax.swing.text.*;

public class SlafRootPaneUI
  extends BasicRootPaneUI
  implements WindowListener, WindowStateListener
{
  private static final String[] KEYS=new String[]
    {
      null,
      "RootPane.frameBorder",
      "RootPane.plainDialogBorder",
      "RootPane.informationDialogBorder",
      "RootPane.errorDialogBorder",
      "RootPane.colorChooserDialogBorder",
      "RootPane.fileChooserDialogBorder",
      "RootPane.questionDialogBorder",
      "RootPane.warningDialogBorder"
    };

  private static final int CORNER_DRAG_WIDTH    =16;
  private static final int BORDER_DRAG_THICKNESS=5;

  private Window              window_;
  private SlafWindowTitlePane tp_;
  private JRootPane           rp_;
  private MouseInputListener  mil_;
  private LayoutManager       layout_;
  private LayoutManager       old_;
  private Cursor              cursor_=
    Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);

  private static RootPaneUI rootPaneUI=new SlafRootPaneUI();

  public static ComponentUI createUI(JComponent _c)
  {
    //System.err.println("CREATE ROOT PANE UI");
    return SlafUtils.jdk()>=1.4
      ? new SlafRootPaneUI()
      : rootPaneUI;
  }

  public void installUI(JComponent _c)
  { 
    super.installUI(_c);
    if(SlafUtils.jdk()>=1.4)
    {
      rp_=(JRootPane)_c;
      //System.err.println("INSTALL ROOT PANE UI "+rp_.getWindowDecorationStyle());
      if(rp_.getWindowDecorationStyle()!=JRootPane.NONE)
        installClientDecorations(rp_);
    }
  }

  public void uninstallUI(JComponent _c)
  {
    super.uninstallUI(_c);
    if(SlafUtils.jdk()>=1.4)
    {
      uninstallClientDecorations(rp_);
      layout_=null;
      mil_   =null;
      rp_  =null;
    }
  }

  private void installBorder(JRootPane _root)
  {
    int style=_root.getWindowDecorationStyle();
    if(style==JRootPane.NONE)
      LookAndFeel.uninstallBorder(_root);
    else
      LookAndFeel.installBorder(_root,"InternalFrame.border");//KEYS[style]);
  }

  private void uninstallBorder(JRootPane _root)
  {
    LookAndFeel.uninstallBorder(_root);
  }

  private void installWindowListeners(JRootPane _root,Component _parent)
  {
    if(_parent instanceof Window)
      window_=(Window)_parent;
    else
      window_=SwingUtilities.getWindowAncestor(_parent);

    if(window_!=null)
    {
      if(mil_==null) mil_=createWindowMouseInputListener(_root);
      window_.addMouseListener      (mil_);
      window_.addMouseMotionListener(mil_);
      window_.addWindowListener     (this);
      window_.addWindowStateListener(this);
    }
  }

  private void uninstallWindowListeners(JRootPane _root)
  {
    if(window_!=null)
    {
      window_.removeMouseListener      (mil_);
      window_.removeMouseMotionListener(mil_);
      window_.removeWindowListener     (this);
      window_.removeWindowStateListener(this);
    }
  }

  private void installLayout(JRootPane _root)
  {
    if(layout_==null) layout_=createLayoutManager();
    old_=_root.getLayout();
    _root.setLayout(layout_);
  }

  private void uninstallLayout(JRootPane _root)
  {
    if(old_!=null)
    {
      _root.setLayout(old_);
      old_=null;
    }
  }

  private void installClientDecorations(JRootPane _root)
  {
    //System.err.println("INSTALL CLIENT DECO");
    installBorder(_root);
    installWindowListeners(_root,_root.getParent());
    setTitlePane(_root,createTitlePane(_root));
    installLayout(_root);

    if(window_!=null)
    {
      _root.revalidate();
      _root.repaint();
    }
  }

  private void uninstallClientDecorations(JRootPane _root)
  {
    uninstallBorder(_root);
    uninstallWindowListeners(_root);
    setTitlePane(_root,null);
    uninstallLayout(_root);

    if(_root.getWindowDecorationStyle()==JRootPane.NONE)
    {
      _root.repaint();
      _root.revalidate();
    }

    if(window_!=null)
      window_.setCursor(Cursor.getPredefinedCursor
                        (Cursor.DEFAULT_CURSOR));

    window_=null;
  }

  private SlafWindowTitlePane createTitlePane(JRootPane _root)
  {
    //System.err.println("CREATE TITLE PANE window="+window_);
    SlafWindowTitlePane r=new SlafWindowTitlePane(_root,this,window_);
    if(mil_!=null)
    {
      r.addMouseListener(mil_);
      r.addMouseMotionListener(mil_);
    }
    return r;
  }

  private MouseInputListener createWindowMouseInputListener(JRootPane _root)
  {
    return new MIH();
  }

  private LayoutManager createLayoutManager()
  {
    return new LM();
  }

  private void setTitlePane(JRootPane _root,SlafWindowTitlePane _titlepane)
  {
    JLayeredPane        lp =_root.getLayeredPane();
    SlafWindowTitlePane old=getTitlePane();

    if(old!=null)
    {
      old.setVisible(false);
      lp .remove(old);
    }
    if(_titlepane!=null)
    {
      lp.add(_titlepane,JLayeredPane.FRAME_CONTENT_LAYER);
      _titlepane.setVisible(true);
    }
    tp_=_titlepane;
  }

  private SlafWindowTitlePane getTitlePane()
  {
    return tp_;
  }

  private JRootPane getRootPane()
  {
    return rp_;
  }

  public void windowStateChanged(WindowEvent _evt)
  {
    getTitlePane().updateMaxIcon();
  }

  public void windowOpened     (WindowEvent _evt) { }
  public void windowClosed     (WindowEvent _evt) { }
  public void windowClosing    (WindowEvent _evt) { }
  public void windowIconified  (WindowEvent _evt) { }
  public void windowDeiconified(WindowEvent _evt) { }

  public void windowActivated  (WindowEvent _evt)
  {
    rp_.repaint();
  }
  
  public void windowDeactivated(WindowEvent _evt)
  {
    rp_.repaint();
  }

  public void propertyChange(PropertyChangeEvent _evt)
  {
    super.propertyChange(_evt);
        
    String name=_evt.getPropertyName();
    if(name==null) return;
    
    //System.err.println("ROOT PANE PROP NAME="+name);

    if(name.equals("windowDecorationStyle"))
    {
      JRootPane root =(JRootPane)_evt.getSource();
      int       style=root.getWindowDecorationStyle();
      uninstallClientDecorations(root);
      if(style!=JRootPane.NONE) installClientDecorations(root);
    }
    else
    if(name.equals("ancestor"))
    {
      uninstallWindowListeners(rp_);
      if(rp_.getWindowDecorationStyle()!=JRootPane.NONE)
      {
        installWindowListeners(rp_,rp_.getParent());
      }
    }
  } 

  private /*static*/ class LM
    implements LayoutManager2
  {
    public Dimension preferredLayoutSize(Container parent)
    {
      Dimension cpd, mbd, tpd;
      int cpWidth = 0;
      int cpHeight = 0;
      int mbWidth = 0;
      int mbHeight = 0;
      int tpWidth = 0;
      int tpHeight = 0;
      Insets i = parent.getInsets();
      JRootPane root = (JRootPane) parent;
      
      if(root.getContentPane() != null) {
        cpd = root.getContentPane().getPreferredSize();
      } else {
        cpd = root.getSize();
      }
      if (cpd != null) {
        cpWidth = cpd.width;
        cpHeight = cpd.height;
      }
      
      if(root.getMenuBar() != null) {
        mbd = root.getMenuBar().getPreferredSize();
        if (mbd != null) {
          mbWidth = mbd.width;
          mbHeight = mbd.height;
        }
      } 
      
      if (root.getWindowDecorationStyle() != JRootPane.NONE &&
          (root.getUI() instanceof SlafRootPaneUI)) {
        JComponent titlePane = ((SlafRootPaneUI)root.getUI()).
          getTitlePane();
                if (titlePane != null) {
                  tpd = titlePane.getPreferredSize();
                  if (tpd != null) {
                    tpWidth = tpd.width;
                    tpHeight = tpd.height;
                    }
                }
      }

      Dimension d=new Dimension
        (Math.max(Math.max(cpWidth, mbWidth), tpWidth) + i.left + i.right, 
         cpHeight + mbHeight + tpWidth + i.top + i.bottom);

      //System.err.println("ROOT PANE PS="+d);
      return d;
    }

    public Dimension minimumLayoutSize(Container parent)
    {
      /*
      Dimension cpd, mbd, tpd;
      int cpWidth = 0;
      int cpHeight = 0;
      int mbWidth = 0;
      int mbHeight = 0;
      int tpWidth = 0;
      int tpHeight = 0;
      Insets i = parent.getInsets();
      JRootPane root = (JRootPane) parent;
      
      if(root.getContentPane() != null) {
        cpd = root.getContentPane().getMinimumSize();
      } else {
        cpd = root.getSize();
      }
      if (cpd != null) {
        cpWidth = cpd.width;
        cpHeight = cpd.height;
      }
      
      if(root.getMenuBar() != null) {
        mbd = root.getMenuBar().getMinimumSize();
        if (mbd != null) {
          mbWidth = mbd.width;
          mbHeight = mbd.height;
        }
      }            
      if (root.getWindowDecorationStyle() != JRootPane.NONE &&
          (root.getUI() instanceof SlafRootPaneUI)) {
        JComponent titlePane = ((SlafRootPaneUI)root.getUI()).
          getTitlePane();
        if (titlePane != null) {
          tpd = titlePane.getMinimumSize();
          if (tpd != null) {
            tpWidth = tpd.width;
            tpHeight = tpd.height;
          }
        }
      }
      
      return new Dimension
        (Math.max(Math.max(cpWidth, mbWidth), tpWidth) + i.left + i.right, 
         cpHeight + mbHeight + tpWidth + i.top + i.bottom);
      */

      Insets i = parent.getInsets();
      Dimension d=getTitlePane().getPreferredSize();
      d.width=40+i.left+i.right;
      d.height+=i.top+i.bottom;
      return d;
    }

    public Dimension maximumLayoutSize(Container target)
    {
      Dimension cpd, mbd, tpd;
      int cpWidth = Integer.MAX_VALUE;
      int cpHeight = Integer.MAX_VALUE;
      int mbWidth = Integer.MAX_VALUE;
      int mbHeight = Integer.MAX_VALUE;
      int tpWidth = Integer.MAX_VALUE;
      int tpHeight = Integer.MAX_VALUE;
      Insets i = target.getInsets();
      JRootPane root = (JRootPane) target;
        
      if(root.getContentPane() != null) {
        cpd = root.getContentPane().getMaximumSize();
        if (cpd != null) {
          cpWidth = cpd.width;
          cpHeight = cpd.height;
        }
      }
      
      if(root.getMenuBar() != null) {
        mbd = root.getMenuBar().getMaximumSize();
        if (mbd != null) {
          mbWidth = mbd.width;
          mbHeight = mbd.height;
        }
      }
      
      if (root.getWindowDecorationStyle() != JRootPane.NONE &&
          (root.getUI() instanceof SlafRootPaneUI)) {
        JComponent titlePane = ((SlafRootPaneUI)root.getUI()).
          getTitlePane();
        if (titlePane != null)
          {
            tpd = titlePane.getMaximumSize();
            if (tpd != null) {
              tpWidth = tpd.width;
              tpHeight = tpd.height;
            }
          }
      }
      
      int maxHeight = Math.max(Math.max(cpHeight, mbHeight), tpHeight);
      // Only overflows if 3 real non-MAX_VALUE heights, sum to > MAX_VALUE
      // Only will happen if sums to more than 2 billion units.  Not likely.
      if (maxHeight != Integer.MAX_VALUE) {
        maxHeight = cpHeight + mbHeight + tpHeight + i.top + i.bottom;
      }
      
      int maxWidth = Math.max(Math.max(cpWidth, mbWidth), tpWidth);
      // Similar overflow comment as above
      if (maxWidth != Integer.MAX_VALUE) {
        maxWidth += i.left + i.right;
      }

      return new Dimension(maxWidth, maxHeight);
    }
    
    public void layoutContainer(Container parent)
    {
      JRootPane root = (JRootPane) parent;
      Rectangle b = root.getBounds();
      Insets i = root.getInsets();
      int nextY = 0;
      int w = b.width - i.right - i.left;
      int h = b.height - i.top - i.bottom;
      
      if(root.getLayeredPane() != null) {
        root.getLayeredPane().setBounds(i.left, i.top, w, h);
      }
      if(root.getGlassPane() != null) {
        root.getGlassPane().setBounds(i.left, i.top, w, h);
      }
      // Note: This is laying out the children in the layeredPane,
      // technically, these are not our children.
      if (root.getWindowDecorationStyle() != JRootPane.NONE &&
          (root.getUI() instanceof SlafRootPaneUI)) {
                JComponent titlePane = ((SlafRootPaneUI)root.getUI()).
                  getTitlePane();
                if (titlePane != null) {
                  Dimension tpd = titlePane.getPreferredSize();
                  if (tpd != null) {
                    int tpHeight = tpd.height;
                    titlePane.setBounds(0, 0, w, tpHeight);
                    nextY += tpHeight;
                  }                    
                }
      }
      if(root.getMenuBar() != null) {
        Dimension mbd = root.getMenuBar().getPreferredSize();
        root.getMenuBar().setBounds(0, nextY, w, mbd.height);
        nextY += mbd.height;
      }
      if(root.getContentPane() != null) {
        Dimension cpd = root.getContentPane().getPreferredSize();
        root.getContentPane().setBounds(0, nextY, w, 
                                        h < nextY ? 0 : h - nextY);
      }
    }
    
    public void addLayoutComponent(String name, Component comp) {}
    public void removeLayoutComponent(Component comp) {}
    public void addLayoutComponent(Component comp, Object constraints) {}
    public float getLayoutAlignmentX(Container target) { return 0.0f; }
    public float getLayoutAlignmentY(Container target) { return 0.0f; }
    public void invalidateLayout(Container target) {}
  }

  private static final int[] cursorMapping = new int[]
  {
    Cursor.NW_RESIZE_CURSOR, Cursor.NW_RESIZE_CURSOR, Cursor.N_RESIZE_CURSOR,
    Cursor.NE_RESIZE_CURSOR, Cursor.NE_RESIZE_CURSOR,
    Cursor.NW_RESIZE_CURSOR, 0, 0, 0, Cursor.NE_RESIZE_CURSOR,
    Cursor.W_RESIZE_CURSOR, 0, 0, 0, Cursor.E_RESIZE_CURSOR,
    Cursor.SW_RESIZE_CURSOR, 0, 0, 0, Cursor.SE_RESIZE_CURSOR,
    Cursor.SW_RESIZE_CURSOR, Cursor.SW_RESIZE_CURSOR, Cursor.S_RESIZE_CURSOR,
    Cursor.SE_RESIZE_CURSOR, Cursor.SE_RESIZE_CURSOR
  };

  private class MIH
    implements MouseInputListener
  {
    private boolean isMovingWindow;
    private int dragCursor;
    private int dragOffsetX;
    private int dragOffsetY;
    private int dragWidth;
    private int dragHeight;

    public void mousePressed(MouseEvent ev)
    {
      JRootPane rootPane=getRootPane();

      if(rootPane.getWindowDecorationStyle()==JRootPane.NONE)
        return;

      Point  dragWindowOffset=ev.getPoint();

      Window w=window_; //(Window)ev.getSource();
      if(w!=null) w.toFront();

      if(ev.getSource()==getTitlePane())
        dragWindowOffset=
          SwingUtilities.convertPoint(getTitlePane(), dragWindowOffset, w);

      Point convertedDragWindowOffset=
          SwingUtilities.convertPoint(w, dragWindowOffset, getTitlePane());
      
      Frame  f=null;
      Dialog d=null;
      
           if(w instanceof Frame ) f=(Frame)w;
      else if(w instanceof Dialog) d=(Dialog)w;
      
      int frameState=(f!=null) ? f.getExtendedState() : 0;
      
      if((getTitlePane()!=null)&&
         getTitlePane().contains(convertedDragWindowOffset))
      {
        if(((f!=null)&&((frameState & Frame.MAXIMIZED_BOTH) == 0)
            || (d != null))
            && dragWindowOffset.y >= BORDER_DRAG_THICKNESS
            && dragWindowOffset.x >= BORDER_DRAG_THICKNESS
            && dragWindowOffset.x < w.getWidth()
            - BORDER_DRAG_THICKNESS)
          {
            isMovingWindow = true;
            dragOffsetX = dragWindowOffset.x;
            dragOffsetY = dragWindowOffset.y;
          }
      }
      else if (f != null && f.isResizable()
               && ((frameState & Frame.MAXIMIZED_BOTH) == 0)
               || (d != null && d.isResizable())) {
        dragOffsetX = dragWindowOffset.x;
        dragOffsetY = dragWindowOffset.y;
        dragWidth = w.getWidth();
        dragHeight = w.getHeight();
        dragCursor = getCursor
          (calculateCorner
           (w, dragWindowOffset.x, dragWindowOffset.y));
      }
    }

    public void mouseReleased(MouseEvent ev) {
      if (dragCursor != 0 && window_ != null && !window_.isValid()) {
        // Some Window systems validate as you resize, others won't,
        // thus the check for validity before repainting.
        window_.validate();
        getRootPane().repaint();
      }
      isMovingWindow = false;
      dragCursor = 0;
    }
    
    public void mouseMoved(MouseEvent ev) {
      JRootPane root = getRootPane();
      
      if (root.getWindowDecorationStyle() == JRootPane.NONE) {
        return;
      }
      
      Window w = window_;//(Window)ev.getSource();
      
      Frame f = null;
      Dialog d = null;
      
      if (w instanceof Frame) {
        f = (Frame)w;
      } else if (w instanceof Dialog) {
        d = (Dialog)w;
      }
      
      // Update the cursor
      int cursor = getCursor(calculateCorner(w, ev.getX(), ev.getY()));

      if (cursor != 0 &&
          ((f != null &&
            (f.isResizable() &&
             (f.getExtendedState() & Frame.MAXIMIZED_BOTH) == 0))
           || (d != null && d.isResizable()))) {
        w.setCursor(Cursor.getPredefinedCursor(cursor));
      }
      else {
        w.setCursor(cursor_);
      }
    }
    
    private void adjust(Rectangle bounds, Dimension min, int deltaX,
                        int deltaY, int deltaWidth, int deltaHeight) {
      bounds.x += deltaX;
      bounds.y += deltaY;
      bounds.width += deltaWidth;
      bounds.height += deltaHeight;

      if (min != null) {
        if (bounds.width < min.width) {
          int correction = min.width - bounds.width;
          if (deltaX != 0) {
            bounds.x -= correction;
          }
          bounds.width = min.width;
        }
        if (bounds.height < min.height) {
          int correction = min.height - bounds.height;
          if (deltaY != 0) {
            bounds.y -= correction;
          }
          bounds.height = min.height;
        }
      }
    }
    
    public void mouseDragged(MouseEvent ev) {
      Window w = window_;//(Window)ev.getSource();
      Point pt = ev.getPoint();
      
      if (isMovingWindow) {
        Point windowPt = w.getLocationOnScreen();
        windowPt.x += pt.x - dragOffsetX;
        windowPt.y += pt.y - dragOffsetY;
        w.setLocation(windowPt);
      }
      else if (dragCursor != 0) {
        Rectangle r = w.getBounds();
        Rectangle startBounds = new Rectangle(r);
        Dimension min = w.getMinimumSize();
        
        switch (dragCursor) {
        case Cursor.E_RESIZE_CURSOR:
          adjust(r, min, 0, 0, pt.x + (dragWidth - dragOffsetX) -
                 r.width, 0);
          break;
        case Cursor.S_RESIZE_CURSOR:
          adjust(r, min, 0, 0, 0, pt.y + (dragHeight - dragOffsetY) -
                 r.height);
          break;
        case Cursor.N_RESIZE_CURSOR:
          adjust(r, min, 0, pt.y -dragOffsetY, 0,
                 -(pt.y - dragOffsetY));
          break;
        case Cursor.W_RESIZE_CURSOR:
          adjust(r, min, pt.x - dragOffsetX, 0,
                 -(pt.x - dragOffsetX), 0);
          break;
        case Cursor.NE_RESIZE_CURSOR:
          adjust(r, min, 0, pt.y - dragOffsetY,
                 pt.x + (dragWidth - dragOffsetX) - r.width,
                 -(pt.y - dragOffsetY));
          break;
        case Cursor.SE_RESIZE_CURSOR:
          adjust(r, min, 0, 0, 
                 pt.x + (dragWidth - dragOffsetX) - r.width,
                 pt.y + (dragHeight - dragOffsetY) -
                 r.height);
          break;
        case Cursor.NW_RESIZE_CURSOR:
          adjust(r, min, pt.x - dragOffsetX,
                 pt.y - dragOffsetY,
                 -(pt.x - dragOffsetX),
                 -(pt.y - dragOffsetY));
          break;
        case Cursor.SW_RESIZE_CURSOR:
          adjust(r, min, pt.x - dragOffsetX, 0,
                 -(pt.x - dragOffsetX),
                 pt.y + (dragHeight - dragOffsetY) - r.height);
          break;
        default:
          break;
        }
        if (!r.equals(startBounds)) {
          w.setBounds(r);
          // Defer repaint/validate on mouseReleased unless dynamic
          // layout is active.
          if (Toolkit.getDefaultToolkit().isDynamicLayoutActive()) {
            w.validate();
            getRootPane().repaint();
          }
        }
      }
    }
    
    public void mouseEntered(MouseEvent ev) {
      Window w = window_;//(Window)ev.getSource();
      cursor_ = w.getCursor();
      mouseMoved(ev);
    }
    
    public void mouseExited(MouseEvent ev) {
      Window w = window_;//(Window)ev.getSource();
      w.setCursor(cursor_);
    }
    
    public void mouseClicked(MouseEvent ev) {
      /*
      Window w = window_;//(Window)ev.getSource();
      Frame f = null;
      
      if (w instanceof Frame) {
        f = (Frame)w;
      } else {
        return;
      }
      
      Point convertedPoint = SwingUtilities.convertPoint(
                                                         w, ev.getPoint(), getTitlePane());
      
      int state = f.getExtendedState();
      if (getTitlePane() != null &&
          getTitlePane().contains(convertedPoint)) {
        if ((ev.getClickCount() % 2) == 0 &&
                        ((ev.getModifiers() & InputEvent.BUTTON1_MASK) != 0)) {
          if (f.isResizable()) {
            if ((state & Frame.MAXIMIZED_BOTH) != 0) {
              f.setExtendedState(state & ~Frame.MAXIMIZED_BOTH);
            }
            else {
              f.setExtendedState(state | Frame.MAXIMIZED_BOTH);
            }
            return;
          }
        }
      }
      */
    }
    
    /**
     * Returns the corner that contains the point <code>x</code>,
     * <code>y</code>, or -1 if the position doesn't match a corner.
     */
    private int calculateCorner(Component c, int x, int y) {
      int xPosition = calculatePosition(x, c.getWidth());
      int yPosition = calculatePosition(y, c.getHeight());
      
      if (xPosition == -1 || yPosition == -1) {
        return -1;
      }
      return yPosition * 5 + xPosition;
    }
    
    /**
     * Returns the Cursor to render for the specified corner. This returns
     * 0 if the corner doesn't map to a valid Cursor
     */
    private int getCursor(int corner) {
      if (corner == -1) {
        return 0;
      }
      return cursorMapping[corner];
    }
    
    /**
     * Returns an integer indicating the position of <code>spot</code>
     * in <code>width</code>. The return value will be:
     * 0 if < BORDER_DRAG_THICKNESS
     * 1 if < CORNER_DRAG_WIDTH
     * 2 if >= CORNER_DRAG_WIDTH && < width - BORDER_DRAG_THICKNESS
     * 3 if >= width - CORNER_DRAG_WIDTH
     * 4 if >= width - BORDER_DRAG_THICKNESS
     * 5 otherwise
     */
    private int calculatePosition(int spot, int width) {
      if (spot < BORDER_DRAG_THICKNESS) {
        return 0;
      }
      if (spot < CORNER_DRAG_WIDTH) {
        return 1;
      }
      if (spot >= (width - BORDER_DRAG_THICKNESS)) {
        return 4;
      }
      if (spot >= (width - CORNER_DRAG_WIDTH)) {
        return 3;
      }
      return 2;
    }
  }
  
  protected void installDefaults(JRootPane _rp)
  {
    super.installDefaults(_rp);
    SlafUtils.setOpaque(_rp);
    LookAndFeel.installColorsAndFont
      (_rp,"RootPane.background","RootPane.background","RootPane.font");
    //LookAndFeel.installBorder(_rp,"RootPane.border");
    //_rp.addPropertyChangeListener(this);
  }

  protected void uninstallDefaults(JRootPane _rp)
  {
    //_rp.removePropertyChangeListener(this);
    //LookAndFeel.uninstallBorder(_rp);
    super.uninstallDefaults(_rp);
  }

  /*
  public void propertyChange(PropertyChangeEvent _evt)
  {
    String name=_evt.getPropertyName();

    if("ancestor".equals(name))
    {
      JRootPane c=(JRootPane)_evt.getSource();
      Border    b=c.getBorder();
      if((b==null)||(b instanceof BorderUIResource))
      {
	b=SlafManager.getBorder("RootPane.border");

	Container p=c.getParent();
        if(p instanceof JFrame)
	{
	  b=new BorderUIResource
	    (new SlafRootPaneBorder
	     ("Frame"));
	}
        else
	if(p instanceof JDialog)
	{
	  b=new BorderUIResource
	    (new SlafRootPaneBorder
	     ("Dialog"));
	}

	c.setBorder(b);
      }
    }

    super.propertyChange(_evt);
  }
  */

  protected void paintBackground(Graphics g, JComponent b)
  {
    if(b.getParent() instanceof Window)
    {
      g.setColor(b.getBackground());
      g.fillRect(0,0,b.getWidth(),b.getHeight());
    }

    if(b.isOpaque())
    {
      Rectangle r=SlafUtils.getFullRect(b);
      Color     c=SlafUtils.getBackground(b);
      SlafUtils.paintBackground(g,r,b,c);
    }
  }

  public void update(Graphics g, JComponent b)
  {
    paintBackground(g,b);
    paint(g,b);
  }
}
