
package com.memoire.slaf;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.text.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
//import javax.swing.plaf.basic.*;

public class SlafSpinnerUI extends SpinnerUI
{
  protected JSpinner spinner;  
  
  private static final PropertyChangeListener propertyChangeListener
    =new PropertyChangeHandler();

  private static final ArrowButtonHandler nextButtonHandler
    =new ArrowButtonHandler(true);
  private static final ArrowButtonHandler previousButtonHandler
    =new ArrowButtonHandler(false);

  private static final Dimension ZERO_SIZE=new Dimension(0,0);

  public static ComponentUI createUI(JComponent c) {
    return new SlafSpinnerUI();
  }

  private void maybeAdd(Component c,String s)
  {
    if(c!=null)
      spinner.add(c,s);
  }

  public void installUI(JComponent c)
  {
    this.spinner=(JSpinner)c;
    installDefaults();
    installListeners();
    maybeAdd(createNextButton()    ,"Next");
    maybeAdd(createPreviousButton(),"Previous");
    maybeAdd(createEditor()        ,"Editor");
    updateEnabledState();
    installKeyboardActions();

    SlafUtils.setOpaque(c);
  }

  public void uninstallUI(JComponent c)
  {
    uninstallDefaults();
    uninstallListeners();
    this.spinner=null;
    c.removeAll();
  }

  protected void installListeners()
  {
    spinner.addPropertyChangeListener(propertyChangeListener);
  }

  protected void uninstallListeners()
  {
    spinner.removePropertyChangeListener(propertyChangeListener);
  }

  protected void installDefaults()
  {
    spinner.setLayout(createLayout());
    LookAndFeel.installBorder(spinner,"Spinner.border");
    LookAndFeel.installColorsAndFont
      (spinner,"Spinner.background","Spinner.foreground","Spinner.font");
  }

  protected void uninstallDefaults()
  {
    spinner.setLayout(null);
  }

  protected LayoutManager createLayout()
  {
    return new SpinnerLayout();
  }

  protected PropertyChangeListener createPropertyChangeListener()
  {
    return new PropertyChangeHandler();
  }

  protected Component createPreviousButton()
  {
    JButton b=new SlafScrollButton(SwingConstants.SOUTH,0,false)
    {
      public Border getBorder()
      {
	return SlafManager.getBorder
	  ("Spinner.previousBorder",super.getBorder());
      }
      public Icon getIcon()
      {
	return SlafManager.getIcon
	  ("Spinner.previousIcon",super.getIcon());
      }
    };
    b.setMargin(new Insets(0,0,0,0));
    b.addActionListener(previousButtonHandler);
    b.addMouseListener (previousButtonHandler);
    return b;
  }

  protected Component createNextButton()
  {
    JButton b=new SlafScrollButton(SwingConstants.NORTH,0,false)
    {
      public Border getBorder()
      {
	return SlafManager.getBorder
	  ("Spinner.nextBorder",super.getBorder());
      }
      public Icon getIcon()
      {
	return SlafManager.getIcon
	  ("Spinner.nextIcon",super.getIcon());
      }
    };
    b.setMargin(new Insets(0,0,0,0));
    b.addActionListener(nextButtonHandler);
    b.addMouseListener (nextButtonHandler);
    return b;
  }

  protected JComponent createEditor()
  {
    return spinner.getEditor();
  }

  protected void replaceEditor(JComponent oldEditor,JComponent newEditor)
  {
    spinner.remove(oldEditor);
    spinner.add(newEditor,"Editor");
  }

  private void updateEnabledState()
  {
    updateEnabledState(spinner,spinner.isEnabled());
  }

  private void updateEnabledState(Container c,boolean enabled)
  {
    for(int counter=c.getComponentCount() - 1; counter >= 0;counter--)
    {
      Component child=c.getComponent(counter);
      
      child.setEnabled(enabled);
      if(child instanceof Container)
	updateEnabledState((Container)child,enabled);
    }
  }

  private void installKeyboardActions() {
    InputMap iMap=getInputMap(JComponent.
				WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
    
    SwingUtilities.replaceUIInputMap(spinner,JComponent.
				     WHEN_ANCESTOR_OF_FOCUSED_COMPONENT,
				     iMap);
  }

  private InputMap getInputMap(int condition) {
    if (condition == JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT) {
      return (InputMap)SlafManager.getObject("Spinner.ancestorInputMap");
    }
    return null;
  }
  
  private static class ArrowButtonHandler
    extends MouseAdapter
    implements ActionListener 
  {
    final javax.swing.Timer autoRepeatTimer;
    final boolean isNext;
    JSpinner spinner=null;
    
    ArrowButtonHandler(boolean isNext) {
      this.isNext=isNext;
      autoRepeatTimer=new javax.swing.Timer(120,this);
      autoRepeatTimer.setInitialDelay(30);
    }

    private JSpinner eventToSpinner(AWTEvent e) {
      Object src=e.getSource();
      while ((src instanceof Component) && !(src instanceof JSpinner)) {
	src=((Component)src).getParent();
      }
      return (src instanceof JSpinner) ? (JSpinner)src : null;
    }
    
    public void actionPerformed(ActionEvent e) {
      if (spinner != null) {
	try {
	  spinner.commitEdit();
	  Object value=(isNext) ? spinner.getNextValue() :
	    spinner.getPreviousValue();
	  if (value != null) {
	    spinner.setValue(value);
	  }
	} catch (IllegalArgumentException iae) {
	  UIManager.getLookAndFeel().provideErrorFeedback(spinner);
	} catch (ParseException pe) {
	  UIManager.getLookAndFeel().provideErrorFeedback(spinner);
	}
      }
    }
    
    public void mousePressed(MouseEvent e) {
      if (SwingUtilities.isLeftMouseButton(e) && e.getComponent().isEnabled()) {
	spinner=eventToSpinner(e);
	autoRepeatTimer.start();
      }
    }
    
    public void mouseReleased(MouseEvent e) {
      autoRepeatTimer.stop();	    
      spinner=null;
    }
  }

  private static class SpinnerLayout implements LayoutManager 
  {
    private Component nextButton=null;
    private Component previousButton=null;
    private Component editor=null;
    
    public void addLayoutComponent(String name,Component c) {
      if ("Next".equals(name)) {
	nextButton=c;
      }
      else if ("Previous".equals(name)) {
	previousButton=c;
      }
      else if ("Editor".equals(name)) {
	editor=c;
      }
    }

    public void removeLayoutComponent(Component c) {
      if (c == nextButton) {
	c=null;
      }
      else if (c == previousButton) {
	previousButton=null;
      }
      else if (c == editor) {
	editor=null;
      }
    }
    
    private Dimension preferredSize(Component c) {
      return (c == null) ? ZERO_SIZE : c.getPreferredSize();
    }

    public Dimension preferredLayoutSize(Container parent) {
      Dimension nextD    =preferredSize(nextButton);
      Dimension previousD=preferredSize(previousButton);
      Dimension editorD  =preferredSize(editor);
      
      //Force the editors height to be a multiple of 2
      //editorD.height=((editorD.height+1) / 2) * 2;     
      Dimension size=new Dimension(editorD.width,editorD.height);

      int type=SlafManager.getInteger("Spinner.type",0);
      int gap =SlafManager.getInteger("Spinner.gap" ,0);

      switch(type)
      {
      case  1:
	size.width+=nextD.width+previousD.width+gap;
	break;
      case 2:
	size.width+=nextD.width+previousD.width+2*gap;
	break;
      default: 
	size.width+=Math.max(nextD.width,previousD.width)+gap;
	break;
      }

      Insets insets=parent.getInsets();
      size.width += insets.left+insets.right;
      size.height += insets.top+insets.bottom;
      return size;
    }
    
    public Dimension minimumLayoutSize(Container parent)
    {
      return preferredLayoutSize(parent);
    }
    
    private void setBounds(Component c,int x,int y,int width,int height)
    {
      if(c!=null)
	c.setBounds(x,y,width,height);
    }
    
    public void layoutContainer(Container parent)
    {
      Insets    insets        =parent.getInsets();
      int       availWidth    =parent.getWidth()-(insets.left+insets.right);
      int       availHeight   =parent.getHeight()-(insets.top+insets.bottom);
      Dimension nextD         =preferredSize(nextButton);
      Dimension previousD     =preferredSize(previousButton);	    
      int       gap           =SlafManager.getInteger("Spinner.gap" ,0);
      int       type          =SlafManager.getInteger("Spinner.type",0);

      int editorW;
      switch(type)
      {
      case 1:
      case 3:
	editorW=availWidth-gap-nextD.width-previousD.width;
	break;
      case 2:
      case 4:
	editorW=availWidth-2*gap-nextD.width-previousD.width;
	break;
      default:
	editorW=availWidth-gap-Math.max(nextD.width,previousD.width);
	break;
      }

      int editorX,nextX,previousX;
      switch(type)
      {
      case 1:
      case 3:
	editorX  =insets.left;
	nextX    =editorX+editorW+gap;
	previousX=nextX+nextD.width;
	break;
      case 2:
      case 4:
	previousX=insets.left;
	editorX  =previousX+previousD.width+gap;
	nextX    =editorX+editorW+gap;
	break;
      default:
	editorX  =insets.left;
	previousX=editorX+editorW+gap;
	nextX    =previousX;
	break;
      }

      setBounds(editor,editorX,insets.top,editorW,availHeight);

      int yt,bw,ht;

      switch(type)
      {
      case 1:
      case 2:
        setBounds(nextButton    ,nextX    ,insets.top,nextD.width    ,availHeight);
	setBounds(previousButton,previousX,insets.top,previousD.width,availHeight);
	break;
      case 3:
      case 4:
	ht=SlafUtils.min(availHeight,nextD.height,previousD.height);
	yt=insets.top+Math.max(0,(availHeight-ht)/2);
        setBounds(nextButton    ,nextX    ,yt,nextD.width    ,ht);
	setBounds(previousButton,previousX,yt,previousD.width,ht);
	break;
      default:
	yt=insets.top+Math.max(0,availHeight/2-nextD.height);
	bw=Math.max(nextD.width,previousD.width);
	if(nextD.height+previousD.height>availHeight)
        {
	  int ov=SlafManager.getInteger("Spinner.overlay",0);
	  int nh=availHeight/2;
	  setBounds(nextButton    ,nextX    ,yt                  ,bw,nh+ov);
	  setBounds(previousButton,previousX,yt+availHeight-nh-ov,bw,nh+ov);
	}
	else
	{
	  setBounds(nextButton    ,nextX    ,yt             ,bw,nextD.height);
	  setBounds(previousButton,previousX,yt+nextD.height,bw,previousD.height);
	}
	break;
      }
    }
  }
  
  private static class PropertyChangeHandler
    implements PropertyChangeListener
  {
    public void propertyChange(PropertyChangeEvent e)
    {
      String    propertyName=e.getPropertyName();
      JSpinner  spinner     =(JSpinner)(e.getSource());
      SpinnerUI spinnerUI   =spinner.getUI();
      
      if(spinnerUI instanceof SlafSpinnerUI)
      {
	SlafSpinnerUI ui=(SlafSpinnerUI)spinnerUI;
	
	if("editor".equals(propertyName))
        {
	  JComponent oldEditor=(JComponent)e.getOldValue();
	  JComponent newEditor=(JComponent)e.getNewValue();
	  ui.replaceEditor(oldEditor,newEditor);
	  ui.updateEnabledState();
	}
	else
	if("enabled".equals(propertyName))
	{
	  ui.updateEnabledState();
	}
      }
    }
  }
}

