/**
 * @modification $Date: 2004/01/26 18:17:34 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafSplitPaneDivider.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class SlafSplitPaneDivider extends BasicSplitPaneDivider
{
  protected RolloverListener rolloverListener;
  protected boolean          isRollover;
  protected boolean          isPressed;

  public SlafSplitPaneDivider(BasicSplitPaneUI ui)
  {
    super(ui);
    FlowLayout lo=new FlowLayout(FlowLayout.LEFT);
    lo.setHgap(0);
    lo.setVgap(0);
    setLayout(lo);
    isRollover=false;
    isPressed =false;

    if(SlafManager.getBoolean
       ("SplitPane.rollover",SlafManager.getBoolean
	("Theme.rollover",true)))
    {
      rolloverListener=new RolloverListener();
      addMouseListener(rolloverListener);
    }

    Color fg=SlafManager.getColor("Divider.foreground");
    if(fg!=null) setForeground(fg);

    Color bg=SlafManager.getColor("Divider.background");
    if(bg!=null) setForeground(bg);
  }

  public Color getBackground()
  {
    Color r=super.getBackground();
    
    if(isEnabled())
    {
           if(isPressed ) r=SlafUtils.darken  (r,1);
      else if(isRollover) r=SlafUtils.brighten(r,1);
    }

    /*
    else
    if(!isOpaque()&&(getParent()!=null))
      r=getParent().getBackground();
    */

    return r;
  }

  public Color getForeground()
  {
    Color r=super.getForeground();
    
    if(isEnabled())
    {
           if(isPressed ) r=SlafUtils.darken  (r,1);
      else if(isRollover) r=SlafUtils.brighten(r,1);
    }

    return r;
  }

  protected void paintBackground(Graphics _g)
  {
    //Rectangle r=SlafUtils.getFullRect(this);
    Dimension size=getSize();
    Rectangle r=new Rectangle(0,0,size.width,size.height);
    Color     c=getBackground();
    SlafUtils.paintBackground(_g,r,splitPane,c,SlafUtils.ENABLED,"SplitPaneDivider");
    /*
    Color bg=getBackground();
    Rectangle r=new Rectangle(0,0,getSize().width,getSize().height);
    _g.setColor(bg);
    _g.fillRect(r.x,r.y,r.width,r.height);
    */
  }
  
  private int d=SlafManager.getInteger("SplitPane.dividerSize",0)%2;

  public void paint(Graphics _g)
  {
    paintBackground(_g);

    //super.paintComponent(_g);

    Dimension size=getSize();

    int w=size.width;
    int h=size.height;
    int x=w/2-2;
    int y=h/2-2;
    Color fg=getForeground();
    Color bg=getBackground();

    boolean isHorizontal=(orientation==JSplitPane.HORIZONTAL_SPLIT);

    /*
    if(isOpaque())
    {
      _g.setColor(bg);
      _g.fillRect(0,0,w,h);
    }
    */

    //_g.setColor(Color.yellow);
    //_g.fillRect(0,0,w,h);

    String p=SlafManager.getString("SplitPane.dividerDrawing");
    //System.err.println("DRAWING="+p);

    if("single".equals(p)||(p==null))
    {
      _g.setColor(bg.darker());
      if(isHorizontal)
      {
	_g.drawLine(x+1,0,x+1,h-1);
	//if(d==-1) d=h%2;
      }
      else
      {
	_g.drawLine(0,y+1,w-1,y+1);
	//if(d==-1) d=w%2;
      }

      if(d==1)
      {
	_g.setColor(Color.black);//bg.darker().darker());
	if(isHorizontal)
	  _g.drawLine(x+2,0,x+2,h-1);
	else
	  _g.drawLine(0,y+2,w-1,y+2);
      }
      _g.setColor(bg.brighter());
      if(isHorizontal)
        _g.drawLine(x+2+d,0,x+2+d,h-1);
      else
        _g.drawLine(0,y+2+d,w-1,y+2+d);
    }
    else
    if("double".equals(p)||"double_raised_line".equals(p))
    {
      _g.setColor(bg.darker());
      if(isHorizontal)
      {
	_g.drawLine(x  ,2,x  ,h-5);
        _g.drawLine(x+3,2,x+3,h-5);
      }
      else
      {
        _g.drawLine(2,y  ,w-5,y  );
        _g.drawLine(2,y+3,w-5,y+3);
      }
      _g.setColor(bg.brighter());
      if(isHorizontal)
      {
        _g.drawLine(x+1,2,x+1,h-5);
        _g.drawLine(x+4,2,x+4,h-5);
      }
      else
      {
        _g.drawLine(2,y+1,w-5,y+1);
	_g.drawLine(2,y+4,w-5,y+4);
      }
    }
    else
    if("square".equals(p))
    {
      _g.setColor(fg);
      _g.drawRect(x-1,y-1,6,6);
    }
    else
    if("filled_square".equals(p))
    {
      _g.setColor(fg);
      _g.fillRect(x-1,y-1,5,5);
    }
    else
    if("lowered_square".equals(p))
    {
      _g.setColor(bg);
      _g.draw3DRect(x-1,y-1,6,6,false);
    }
    else
    if("raised_square".equals(p))
    {
      _g.setColor(bg);
      _g.draw3DRect(x-1,y-1,6,6,true);
    }
    else
    if("line".equals(p))
    {
      _g.setColor(fg);
      if(isHorizontal) _g.drawLine(x+2,2,x+2,h-5);
      else             _g.drawLine(2,y+2,w-5,y+2);
    }
    else
    if("thick_line".equals(p))
    {
      _g.setColor(fg);
      if(isHorizontal)
      {
        _g.drawLine(x+1,2,x+1,h-5);
        _g.drawLine(x+2,2,x+2,h-5);
      }
      else
      {
        _g.drawLine(2,y+1,w-5,y+1);
        _g.drawLine(2,y+2,w-5,y+2);
      }
    }
    else
    if("double_line".equals(p))
    {
      _g.setColor(fg);
      if(isHorizontal)
      {
        _g.drawLine(x  ,2,x  ,h-5);
	_g.drawLine(x+3,2,x+3,h-5);
      }
      else
      {
	_g.drawLine(2,y  ,w-5,y  );
        _g.drawLine(2,y+3,w-5,y+3);
      }
    }
    else
    if("".equals(p)||"nothing".equals(p))
    {
    }

    paintComponents(_g);

    if(SlafUtils.jdk()>=1.2)
    {
      Border border=getBorder();
      if(border!=null) border.paintBorder(this,_g,0,0,w,h);
    }
  }

  /*
  public void paint(Graphics _g)
  {
    paintBackground(_g);

    super.paint(_g);

    Color bg=getBackground();
    Dimension sz=getSize();
    int w=sz.width;
    int h=sz.height;
    int p=5;//43;

    if(orientation==JSplitPane.HORIZONTAL_SPLIT)
    {
      int x=w/2-2;

      _g.setColor(SlafUtils.darken(bg));
      _g.drawLine(x  ,p,x  ,h-5);
      _g.drawLine(x+3,p,x+3,h-5);
      _g.setColor(SlafUtils.brighten(bg));
      _g.drawLine(x+1,p,x+1,h-5);
      _g.drawLine(x+4,p,x+4,h-5);
    }
    else
    {
      int y=h/2-2;

      _g.setColor(SlafUtils.darken(bg));
      _g.drawLine(p,y  ,w-5,y  );
      _g.drawLine(p,y+3,w-5,y+3);
      _g.setColor(SlafUtils.brighten(bg));
      _g.drawLine(p,y+1,w-5,y+1);
      _g.drawLine(p,y+4,w-5,y+4);
    }
  }
  */

  private JButton createOneTouchButton(final boolean _left)
  {
    JButton r=new SlafSpecialButton()
    {
      public Icon getIcon()
      {
	if(_left)
	    return (orientation==JSplitPane.HORIZONTAL_SPLIT)
	      ? SlafManager.getIcon
	      ("SplitPane.westIcon",SlafManager.getIcon("ScrollButton.westIcon"))
	      : SlafManager.getIcon
	      ("SplitPane.northIcon",SlafManager.getIcon("ScrollButton.northIcon"));
	else
	    return (orientation==JSplitPane.HORIZONTAL_SPLIT)
	      ? SlafManager.getIcon
	      ("SplitPane.eastIcon",SlafManager.getIcon("ScrollButton.eastIcon"))
	      : SlafManager.getIcon
	      ("SplitPane.southIcon",SlafManager.getIcon("ScrollButton.southIcon"));
      }
    };

    Border b=SlafManager.getBorder("SplitPane.buttonBorder",null);
    if(b!=null) r.setBorder(new CompoundBorder(b,new EmptyBorder(0,0,0,0)));
    r.setPreferredSize(r.getPreferredSize());

    return r;
  }

  protected JButton createLeftOneTouchButton()
  {
      return createOneTouchButton(true);
      /*
      (orientation==JSplitPane.HORIZONTAL_SPLIT
       ? SwingConstants.WEST
       : SwingConstants.NORTH);
      */
  }

  protected JButton createRightOneTouchButton()
  {
      return createOneTouchButton(false);
      /*
      (orientation==JSplitPane.HORIZONTAL_SPLIT
       ? SwingConstants.EAST
       : SwingConstants.SOUTH);
      */
  }

  class RolloverListener extends MouseAdapter
  {
    public void mousePressed(MouseEvent e)
    {
      isPressed=true;
      repaint();
    }

    public void mouseReleased(MouseEvent e)
    {
      isPressed=false;
      repaint();
    }

    public void mouseEntered(MouseEvent e)
    {
      isRollover=true;
      repaint();
    }

    public void mouseExited(MouseEvent e)
    {
      isRollover=false;
      repaint();
    }
  }

  /*
  public void paint(Graphics g)
  {
    super.paint(g);

    Dimension sz=getSize();
    Color     bg=getBackground();
    g.setColor(bg);
    g.draw3DRect(0,0,sz.width-1,sz.height-1,true);
  }
  */

  /*
  public void paint(Graphics g)
  {
    Color bgColor=(SlafUtils.hasFocus(splitPane) ?
      SlafManager.getColor("SplitPane.shadow") :
      getBackground();
    // Color bgColor=Color.

    Dimension size = getSize();

    if(bgColor!=null)
    {
      g.setColor(bgColor);
      g.fillRect(0, 0, size.width, size.height);
    }
    super.paint(g);
  }
  */
}
