/**
 * @modification $Date: 2004/01/26 18:17:34 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafTableCellRenderer.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;


import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.table.*;

import java.awt.*;
import java.text.*;
import java.util.*;
import java.io.Serializable;

public class SlafTableCellRenderer
       extends JLabel 
       implements TableCellRenderer //, Serializable, UIResource
{
  protected boolean      enabled;
  protected boolean      selected;
  protected boolean      focused;
  protected NumberFormat number_formatter=NumberFormat.getInstance();
  protected DateFormat   date_formatter  =DateFormat.getInstance();

  public SlafTableCellRenderer()
  {
    //setOpaque(true);
  }

  protected Border getCellBorder()
  {
    return SlafManager.getBorder
	("TableCell.border",SlafManager.getBorder("Table.cellBorder"));
  }

  protected Border getSelectedCellBorder()
  {
    return SlafManager.getBorder
	("TableCell.selectedBorder",SlafManager.getBorder
	 ("Table.selectedCellBorder"));
  }

  protected Border getFocusCellBorder()
  {
    return SlafManager.getBorder
	("TableCell.focusBorder",SlafManager.getBorder
	 ("Table.focusCellHighlightBorder"));
  }

  public Component getTableCellRendererComponent
    (JTable table,
     Object value,
     boolean isSelected, 
     boolean cellHasFocus,
     int row, int col)
  {
    selected=isSelected;
    enabled =table.isEnabled();
    focused =cellHasFocus;

    setEnabled(enabled);
    setFont(table.getFont());
    setOpaque(true);

    String text  ="";
    Icon   icon  =null;
    int    halign=JLabel.LEFT;

    if(selected)
    {
      setBackground(table.getSelectionBackground());
      setForeground(table.getSelectionForeground());
      setBorder(getSelectedCellBorder());
    }
    else
    {
      setBackground(table.getBackground());
      setForeground(table.getForeground());
      setBorder(getCellBorder());
    }
	
    if(cellHasFocus)
    {
      Color bg=SlafManager.getColor
	  ("Table.focusCell"
	   +(selected ? "Selection" : "")
	   +"Background",null);
      if(bg!=null) setBackground(bg);

      Color fg=SlafManager.getColor
	  ("Table.focusCell"
	   +(selected ? "Selection" : "")
	   +"Foreground",null);
      if(fg!=null) setForeground(fg);

      Border bd=getFocusCellBorder();
      if(bd!=null) setBorder(bd);
    }

    if(value==null)
    {
    }
    else
    if(value instanceof Boolean)
    {
      value=(Boolean.TRUE.equals(value)
	     ? SlafManager.getIcon("CheckBox.selectedIcon")
	     : SlafManager.getIcon("CheckBox.unselectedIcon"));
    }
    else
    if(value instanceof Icon)
    {
      icon=(Icon)value;
    }
    else
    if((value instanceof Double)||(value instanceof Float))
    {
      text=number_formatter.format(value);
    }
    else
    if(value instanceof Date)
    {
      text=date_formatter.format(value);
    }
    else
    {
      text=value.toString();
    }

    if(value instanceof Number)
	halign=JLabel.RIGHT;
    else
    if(value instanceof Icon)
	halign=JLabel.CENTER;

    if((icon==null)&&"".equals(text))
      text=" ";

    setText(text);
    setIcon(icon);
    setHorizontalAlignment(halign);

    return this;
  }

  public void setOpaque(boolean _state)
  {
    super.setOpaque(isOpaque());
  }

  public boolean isOpaque()
  {
    boolean r=true;
    if(selected||focused)
      r=SlafManager.getBoolean("TableCell.selectionOpaque",true);
    else
      r=SlafManager.getBoolean("TableCell.opaque",false);
    //System.err.println("IS_OPAQUE="+r);
    return r;
  }

  protected void paintBackground(Graphics g)
  {
    Rectangle r=SlafUtils.getFullRect  (this);
    Color     c=SlafUtils.getBackground(this);
    int       t=SlafUtils.ENABLED;
    if(selected) t=SlafUtils.SELECTED;
    //if(focused)  t=SlafUtils.PRESSED;
    //if(!enabled) t=SlafUtils.ENABLED;
    SlafUtils.paintBackground(g,r,this,c,t,"TableCell");
  }
  
  protected void paintFocus(Graphics g)
  {
    if(getFocusCellBorder()==null)
    {
      Rectangle r=SlafUtils.getViewRect(this);
      g.setColor(SlafManager.getColor("Table.focus",getForeground()));
      g.drawLine(r.x, r.y+r.height-1, r.x+r.width-1, r.y+r.height-1);
    }
  }

  public void paint(Graphics g)
  {
    paintBackground(g);
    super.paint(g);
    if(focused) paintFocus(g);
    //SlafUtils.paintOver(g,(AbstractButton)c);
  }
}
