/**
 * @modification $Date: 2004/01/26 18:17:34 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafToolBarUI.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.Serializable;
import java.beans.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class SlafToolBarUI extends BasicToolBarUI
{
  private SlafPropertyListener listener;

  public static ComponentUI createUI( JComponent c )
  {
    return new SlafToolBarUI();
  }

  public void installUI(JComponent c)
  {
    super.installUI(c);
    SlafUtils.setOpaque(c);

    if(c.getLayout() instanceof UIResource)
    {
      c.setLayout(createGridLayout());
      ((JToolBar)c).setMargin(new Insets(0,0,0,0));
    }

    listener=new SlafPropertyListener((JToolBar)c);
    c.addPropertyChangeListener(listener);
  }

  public void uninstallUI(JComponent c)
  {
    super.uninstallUI(c);
    c.removePropertyChangeListener(listener);
  }

  protected void paintBackground(Graphics g, JComponent b)
  {
    Rectangle r=SlafUtils.getFullRect(b);
    Color     c=SlafUtils.getBackground(b);
    SlafUtils.paintBackground(g,r,b,c);
  }
  
  public void update(Graphics g, JComponent c)
  {
    paintBackground(g,c);
    paint(g,c);
    paintOver(g,c);
  }

  public void paintOver(Graphics g, JComponent c)
  {
    /*
    if(toolBar.getOrientation()==JToolBar.HORIZONTAL)
    {
      Icon i=SlafManager.getIcon("ToolBar.horizontalIcon");
      if(i!=null) i.paintIcon(c,g,0,(toolBar.getHeight()-i.getIconHeight())/2);
    }
    else
    {
      Icon i=SlafManager.getIcon("ToolBar.verticalIcon");
      if(i!=null) i.paintIcon(c,g,(toolBar.getWidth()-i.getIconWidth())/2,0);
    }
    */
  }

  protected LayoutManager createGridLayout()
  {
    boolean v  =(toolBar.getOrientation()==JToolBar.VERTICAL);
    int     gap=SlafManager.getInteger("ToolBar.gap",2);

    SlafGridLayout lo=new SlafGridLayout
      (v?1:0,gap,gap,false,false,false,false);
    lo.setXalign(v?0.f:1.f);
    lo.setYalign(v?1.f:0.5f);
    lo.setRfilled(false);
    lo.setCfilled(false);
    return lo;
  }

  protected void setBorderToRollover   (Component c) { }
  protected void setBorderToNonRollover(Component c) { }
  protected void setBorderToNormal     (Component c) { }

  protected DockingListener createDockingListener(JToolBar _toolbar)
  {
    return new SlafDockingListener(_toolbar);
  }

  protected void setDragOffset(Point p)
  {
    if(dragWindow==null)
      dragWindow=createDragWindow(toolBar);
    dragWindow.setOffset( p );
  }

  protected class SlafDockingListener extends DockingListener
  {
    protected boolean pressedInBumps = false;

    public SlafDockingListener(JToolBar _toolbar)
    {
      super(_toolbar);
    } 

    public void mousePressed(MouseEvent e)
    { 
      super.mousePressed(e);
      setDragOffset(e.getPoint());
      pressedInBumps=false;

      Rectangle bumpRect=new Rectangle();
      
      if(this.toolBar.getOrientation()==JToolBar.HORIZONTAL)
      //if(this.toolBar.getSize().height<=this.toolBar.getSize().width)
	bumpRect.setBounds(0,0,14,this.toolBar.getSize().height);
      else
	bumpRect.setBounds(0,0,this.toolBar.getSize().width,14);

      if(bumpRect.contains(e.getPoint()))
	pressedInBumps = true;
    }

    public void mouseDragged(MouseEvent e)
    {
      if(pressedInBumps)
	super.mouseDragged(e);
    }
  }

  protected class SlafPropertyListener implements PropertyChangeListener
  {
    private JToolBar toolbar_;

    public SlafPropertyListener(JToolBar _toolbar)
    {
      toolbar_=_toolbar;
    } 

    public void propertyChange(PropertyChangeEvent _evt)
    {
      if(_evt.getPropertyName().equals("orientation"))
      {
	SlafUtils.trace("Change orientation");
	LayoutManager lo=toolbar_.getLayout();
	if(lo instanceof UIResource)
	  toolbar_.setLayout(createGridLayout());
	/*
	{
	  ((SlafGridLayout)lo).setColumns
	    (257-((SlafGridLayout)lo).getColumns());
	  ((SlafGridLayout)lo).invalidateLayout(toolbar_);
	  toolbar_.setSize(toolbar_.getPreferredSize());
	}
	*/

	/*
	Component[] c=toolbar_.getComponents();

	LayoutManager lo=createGridLayout();
	toolbar_.setLayout(lo);

	for(int i=0;i<c.length;i++)
	{
	  if(c[i] instanceof JToolBar.Separator)
	  {
	    JToolBar.Separator s=(JToolBar.Separator)c[i];
	    Dimension size=s.getSeparatorSize();
	    if(size==null||size instanceof UIResource)
	      s.setSeparatorSize(SlafToolBarSeparatorUI.getDefaultSize(s));
	  }
	  toolbar_.add(c[i],i);
	}

	//toolbar_.setPreferredSize(lo.preferredLayoutSize(toolbar_));
	  //(lo.minimumLayoutSize(toolbar_));

	Container p=toolbar_.getParent();
	if(p!=null)
	{
	  p.invalidate();
	  p.doLayout();
	  p.validate();
	}
	*/
      }
    }
  }
}
