/**
 * @modification $Date: 2004/01/26 18:17:34 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafTreeCellRenderer.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;


import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.tree.*;

import java.awt.*;
import java.text.*;
import java.util.*;
import java.io.Serializable;

public class SlafTreeCellRenderer
       extends JLabel
       implements TreeCellRenderer
{
  protected boolean      enabled;
  protected boolean      selected;
  protected boolean      focused;
  //protected NumberFormat number_formatter=NumberFormat.getInstance();
  //protected DateFormat   date_formatter  =DateFormat.getInstance();

  protected transient Icon closedIcon;
  protected transient Icon leafIcon;
  protected transient Icon openIcon;

  public SlafTreeCellRenderer()
  {
    //setHorizontalAlignment(JLabel.LEFT);
    setLeafIcon  (SlafManager.getIcon("Tree.leafIcon"));
    setClosedIcon(SlafManager.getIcon("Tree.closedIcon"));
    setOpenIcon  (SlafManager.getIcon("Tree.openIcon"));
  }

  public Icon getDefaultOpenIcon()        { return openIcon; }
  public Icon getDefaultClosedIcon()      { return closedIcon; }
  public Icon getDefaultLeafIcon()        { return leafIcon; }
  public void setOpenIcon(Icon newIcon)   { openIcon = newIcon; }
  public Icon getOpenIcon()               { return openIcon; }
  public void setClosedIcon(Icon newIcon) { closedIcon = newIcon; }
  public Icon getClosedIcon()             { return closedIcon; }
  public void setLeafIcon(Icon newIcon)   { leafIcon = newIcon; }
  public Icon getLeafIcon()               { return leafIcon; }

  protected Color getNonSelectionForeground()
    { return SlafManager.getColor("Tree.foreground"); }

  protected Color getNonSelectionBackground()
    { return SlafManager.getColor("Tree.background"); }

  protected Color getSelectionForeground()
    { return SlafManager.getColor("Tree.selectionForeground"); }

  protected Color getSelectionBackground()
    { return SlafManager.getColor("Tree.selectionBackground"); }

  protected Border getCellBorder()
  {
    return SlafManager.getBorder
	("TreeCell.border",SlafManager.getBorder
	 ("Tree.cellBorder"));
  }

  protected Border getSelectedCellBorder()
  {
    return SlafManager.getBorder
	("TreeCell.selectedBorder",SlafManager.getBorder
	 ("Tree.selectedCellBorder"));
  }

  protected Border getFocusCellBorder()
  {
    return SlafManager.getBorder
	("TreeCell.focusBorder",SlafManager.getBorder
	 ("Tree.focusCellHighlightBorder"));
  }

  public Component getTreeCellRendererComponent
    (JTree tree, Object value,
     boolean isSelected, boolean isExpanded,
     boolean leaf, int row,
     boolean cellHasFocus)
  {
    Icon   icon=null;
    String text=tree.convertValueToText
      (value,isSelected,isExpanded,leaf,row,cellHasFocus);

    selected=isSelected;
    enabled =tree.isEnabled();
    focused =cellHasFocus;

    setEnabled(enabled);
    setFont(tree.getFont());
    setOpaque(true);

    if(selected)
    {
      setBackground(getSelectionBackground());
      setForeground(getSelectionForeground());
      setBorder(getSelectedCellBorder());
    }
    else
    {
      setBackground(getNonSelectionBackground()); // tree.getBackground());
      setForeground(getNonSelectionForeground()); // tree.getForeground());
      setBorder(getCellBorder());
    }

    if(cellHasFocus)
    {
      Color bg=SlafManager.getColor
	  ("Tree.focusCell"
	   +(selected ? "Selection" : "")
	   +"Background",null);
      if(bg!=null) setBackground(bg);

      Color fg=SlafManager.getColor
	  ("Tree.focusCell"
	   +(selected ? "Selection" : "")
	   +"Foreground",null);
      if(fg!=null) setForeground(fg);

      Border bd=getFocusCellBorder();
      if(bd!=null) setBorder(bd);
    }

         if (leaf)       icon=getLeafIcon();
    else if (isExpanded) icon=getOpenIcon();
    else                 icon=getClosedIcon();
        
    if((icon==null)&&"".equals(text)) text=" ";

    setIcon(icon);
    setText(text);

    return this;
  }

  public void setOpaque(boolean _state)
  {
    super.setOpaque(isOpaque());
  }

  public boolean isOpaque()
  {
    boolean r=true;
    if(selected) r=SlafManager.getBoolean("TreeCell.selectionOpaque",true);
    else         r=SlafManager.getBoolean("TreeCell.opaque",false);
    //System.err.println("IS_OPAQUE="+r);
    return r;
  }

  protected void paintBackground(Graphics g)
  {
    Rectangle r=SlafUtils.getFullRect  (this);
    Color     c=SlafUtils.getBackground(this);
    int       t=SlafUtils.ENABLED;
    if(selected) t=SlafUtils.SELECTED;
    SlafUtils.paintBackground(g,r,this,c,t,"TreeCell");
  }
  
  protected void paintFocus(Graphics g)
  {
    if(getFocusCellBorder()==null)
    {
      Rectangle r=SlafUtils.getViewRect(this);
      g.setColor(SlafManager.getColor("Tree.focus",getForeground()));
      g.drawLine(r.x, r.y+r.height-1, r.x+r.width-1, r.y+r.height-1);
    }
  }

  public void paint(Graphics g)
  {
    paintBackground(g);
    super.paint(g);
    if(focused) paintFocus(g);
    //SlafUtils.paintOver(g,(AbstractButton)c);
  }
}
