/**
 * @modification $Date: 2004/06/25 17:55:53 $
 * @statut       unstable
 * @file         com/memoire/slaf/SlafWindowTitlePane.java
 * @version      0.20
 * @author       Guillaume Desnoix
 * @email        guillaume-desnoix@memoire.com
 * @license      GNU General Public License 2 (GPL2)
 * @copyright    1999-2004 Guillaume Desnoix
 */

package com.memoire.slaf;
import com.memoire.slaf.*;


import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.InternalFrameEvent;
import java.util.EventListener;
import java.beans.*;

class SlafWindowTitlePane
  extends SlafAbstractTitlePane
{
  protected JRootPane  rp_;
  protected Window     frame;

  public SlafWindowTitlePane(JRootPane _rp,SlafRootPaneUI _ui,Window f)
  {
    rp_=_rp;
    frame=f;

    rp_.addPropertyChangeListener(this);
    if(frame!=null) frame.addPropertyChangeListener(this);
  }

  public final boolean isEnabled()
  {
    return true;
  }

  public final boolean isSelected()
  {
    return (frame==null) ? false : frame.isActive();
  }

  public final boolean isClosable()
  {
    if(frame instanceof Frame ) return true;
    if(frame instanceof Dialog) return true;
    return false;
  }

  public final boolean isIconifiable()
  {
    if(frame instanceof Frame ) return true;
    if(frame instanceof Dialog) return false;
    return false;
  }

  public final boolean isLinifiable()
  {
    return false;
    //SlafManager.getBoolean("InternalFrame.linifiable",true);
  }

  public final boolean isResizable()
  {
    if(frame instanceof Frame ) return ((Frame) frame).isResizable();
    if(frame instanceof Dialog) return ((Dialog)frame).isResizable();
    return false;
  }

  public final boolean isMaximizable()
  {
    if(frame instanceof Frame ) return ((Frame) frame).isResizable();
    if(frame instanceof Dialog) return ((Dialog)frame).isResizable();
    return false;
  }

  public final boolean isClosed()
  {
    if(frame instanceof Frame ) return ((Frame) frame).isVisible();
    if(frame instanceof Dialog) return ((Dialog)frame).isVisible();
    return false;
  }

  public final void setClosed(boolean b)
  {
    if(frame!=null)
    {
      if(b)
        frame.dispatchEvent(new WindowEvent(frame,WindowEvent.WINDOW_CLOSING));
    }
    else
    {
      if(b) frame.hide();
      else  frame.show();
    }
  }

  public final boolean isIcon()
  {
    boolean r=false;

    if(SlafUtils.jdk()>=1.4)
    {
      if(frame instanceof Frame)
      {
        int es=((Frame)frame).getExtendedState();
        if((es&Frame.ICONIFIED)!=0) r=true;
      }
    }

    return r;
  }

  public final void setIcon(boolean b)
  {
    if(SlafUtils.jdk()>=1.4)
    {
      if(frame instanceof Frame)
      {
        int es=((Frame)frame).getExtendedState();
        if(b) es|=Frame.ICONIFIED;
        else  es&=~Frame.ICONIFIED;
        ((Frame)frame).setExtendedState(es);
      }
    }
  }

  public final boolean isMaximum()
  {
    boolean r=false;

    if(SlafUtils.jdk()>=1.4)
    {
      if(frame instanceof Frame)
      {
        int es=((Frame)frame).getExtendedState();
        if((es&Frame.MAXIMIZED_BOTH)!=0) r=true;
      }
    }

    return r;
  }

  public final void setMaximum(boolean b)
  {
    if(SlafUtils.jdk()>=1.4)
    {
      if(frame instanceof Frame)
      {
        int es=((Frame)frame).getExtendedState();
        if(b) es|=Frame.MAXIMIZED_BOTH;
        else  es&=~Frame.MAXIMIZED_BOTH;
        ((Frame)frame).setExtendedState(es);
      }
    }
  }

  protected final Color getBackground0()
  {
    Color r=null;
    if(frame!=null) r=frame.getBackground();
    if(r==null)     r=rp_.getBackground();
    return r;
  }

  protected final Color getForeground0()
  {
    Color r=null;
    if(frame!=null) r=frame.getForeground();
    if(r==null)     r=rp_.getForeground();
    return r;
  }

  protected final String getFrameTitle()
  {
    if(frame instanceof Frame ) return ((Frame) frame).getTitle();
    if(frame instanceof Dialog) return ((Dialog)frame).getTitle();
    return null;
  }

  protected final Icon getFrameIcon()
  {
    //frame.getFrameIcon();
    return null;
  }

  protected final Component getFrameComponent()
  {
    return frame;
  }

  protected final Dimension getMinimumFrameSize()
  {
    //Border  b=frame.getBorder();
    Insets    i=frame.getInsets(); //b.getBorderInsets(frame);
    Dimension d=getMinimumSize();
    //SlafUtils.trace("MIN SIZE="+d);
    Dimension r=new Dimension(120+i.left+i.right,d.height+i.top+i.bottom);
    return r;
  }

  protected final boolean isHidden()
  {
    return (frame!=null)&&frame.isShowing()&&!isVisible();
  }

  public final boolean isPalette()
  {
    return false;
  }

  public void setPalette(boolean b)
  {
  }
}
